"""
Python module generated from Java source file java.lang.invoke.MethodHandle

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.constant import ClassDesc
from java.lang.constant import Constable
from java.lang.constant import DirectMethodHandleDesc
from java.lang.constant import MethodHandleDesc
from java.lang.constant import MethodTypeDesc
from java.lang.invoke import *
from java.util import Arrays
from java.util import Objects
from java.util import Optional
from java.util.function import Function
from jdk.internal.vm.annotation import IntrinsicCandidate
from typing import Any, Callable, Iterable, Tuple


class MethodHandle(Constable):
    """
    A method handle is a typed, directly executable reference to an underlying method,
    constructor, field, or similar low-level operation, with optional
    transformations of arguments or return values.
    These transformations are quite general, and include such patterns as
    .asType conversion,
    .bindTo insertion,
    java.lang.invoke.MethodHandles.dropArguments deletion,
    and java.lang.invoke.MethodHandles.filterArguments substitution.
    
    <h2>Method handle contents</h2>
    Method handles are dynamically and strongly typed according to their parameter and return types.
    They are not distinguished by the name or the defining class of their underlying methods.
    A method handle must be invoked using a symbolic type descriptor which matches
    the method handle's own .type() type descriptor.
    
    Every method handle reports its type descriptor via the .type() type accessor.
    This type descriptor is a java.lang.invoke.MethodType MethodType object,
    whose structure is a series of classes, one of which is
    the return type of the method (or `void.class` if none).
    
    A method handle's type controls the types of invocations it accepts,
    and the kinds of transformations that apply to it.
    
    A method handle contains a pair of special invoker methods
    called .invokeExact invokeExact and .invoke invoke.
    Both invoker methods provide direct access to the method handle's
    underlying method, constructor, field, or other operation,
    as modified by transformations of arguments and return values.
    Both invokers accept calls which exactly match the method handle's own type.
    The plain, inexact invoker also accepts a range of other call types.
    
    Method handles are immutable and have no visible state.
    Of course, they can be bound to underlying methods or data which exhibit state.
    With respect to the Java Memory Model, any method handle will behave
    as if all of its (internal) fields are final variables.  This means that any method
    handle made visible to the application will always be fully formed.
    This is True even if the method handle is published through a shared
    variable in a data race.
    
    Method handles cannot be subclassed by the user.
    Implementations may (or may not) create internal subclasses of `MethodHandle`
    which may be visible via the java.lang.Object.getClass Object.getClass
    operation.  The programmer should not draw conclusions about a method handle
    from its specific class, as the method handle class hierarchy (if any)
    may change from time to time or across implementations from different vendors.
    
    <h2>Method handle compilation</h2>
    A Java method call expression naming `invokeExact` or `invoke`
    can invoke a method handle from Java source code.
    From the viewpoint of source code, these methods can take any arguments
    and their result can be cast to any return type.
    Formally this is accomplished by giving the invoker methods
    `Object` return types and variable arity `Object` arguments,
    but they have an additional quality called *signature polymorphism*
    which connects this freedom of invocation directly to the JVM execution stack.
    
    As is usual with virtual methods, source-level calls to `invokeExact`
    and `invoke` compile to an `invokevirtual` instruction.
    More unusually, the compiler must record the actual argument types,
    and may not perform method invocation conversions on the arguments.
    Instead, it must generate instructions that push them on the stack according
    to their own unconverted types.  The method handle object itself is pushed on
    the stack before the arguments.
    The compiler then generates an `invokevirtual` instruction that invokes
    the method handle with a symbolic type descriptor which describes the argument
    and return types.
    
    To issue a complete symbolic type descriptor, the compiler must also determine
    the return type.  This is based on a cast on the method invocation expression,
    if there is one, or else `Object` if the invocation is an expression,
    or else `void` if the invocation is a statement.
    The cast may be to a primitive type (but not `void`).
    
    As a corner case, an uncasted `null` argument is given
    a symbolic type descriptor of `java.lang.Void`.
    The ambiguity with the type `Void` is harmless, since there are no references of type
    `Void` except the null reference.
    
    <h2>Method handle invocation</h2>
    The first time an `invokevirtual` instruction is executed
    it is linked by symbolically resolving the names in the instruction
    and verifying that the method call is statically legal.
    This also holds for calls to `invokeExact` and `invoke`.
    In this case, the symbolic type descriptor emitted by the compiler is checked for
    correct syntax, and names it contains are resolved.
    Thus, an `invokevirtual` instruction which invokes
    a method handle will always link, as long
    as the symbolic type descriptor is syntactically well-formed
    and the types exist.
    
    When the `invokevirtual` is executed after linking,
    the receiving method handle's type is first checked by the JVM
    to ensure that it matches the symbolic type descriptor.
    If the type match fails, it means that the method which the
    caller is invoking is not present on the individual
    method handle being invoked.
    
    In the case of `invokeExact`, the type descriptor of the invocation
    (after resolving symbolic type names) must exactly match the method type
    of the receiving method handle.
    In the case of plain, inexact `invoke`, the resolved type descriptor
    must be a valid argument to the receiver's .asType asType method.
    Thus, plain `invoke` is more permissive than `invokeExact`.
    
    After type matching, a call to `invokeExact` directly
    and immediately invoke the method handle's underlying method
    (or other behavior, as the case may be).
    
    A call to plain `invoke` works the same as a call to
    `invokeExact`, if the symbolic type descriptor specified by the caller
    exactly matches the method handle's own type.
    If there is a type mismatch, `invoke` attempts
    to adjust the type of the receiving method handle,
    as if by a call to .asType asType,
    to obtain an exactly invokable method handle `M2`.
    This allows a more powerful negotiation of method type
    between caller and callee.
    
    (*Note:* The adjusted method handle `M2` is not directly observable,
    and implementations are therefore not required to materialize it.)
    
    <h2>Invocation checking</h2>
    In typical programs, method handle type matching will usually succeed.
    But if a match fails, the JVM will throw a WrongMethodTypeException,
    either directly (in the case of `invokeExact`) or indirectly as if
    by a failed call to `asType` (in the case of `invoke`).
    
    Thus, a method type mismatch which might show up as a linkage error
    in a statically typed program can show up as
    a dynamic `WrongMethodTypeException`
    in a program which uses method handles.
    
    Because method types contain "live" `Class` objects,
    method type matching takes into account both type names and class loaders.
    Thus, even if a method handle `M` is created in one
    class loader `L1` and used in another `L2`,
    method handle calls are type-safe, because the caller's symbolic type
    descriptor, as resolved in `L2`,
    is matched against the original callee method's symbolic type descriptor,
    as resolved in `L1`.
    The resolution in `L1` happens when `M` is created
    and its type is assigned, while the resolution in `L2` happens
    when the `invokevirtual` instruction is linked.
    
    Apart from type descriptor checks,
    a method handle's capability to call its underlying method is unrestricted.
    If a method handle is formed on a non-public method by a class
    that has access to that method, the resulting handle can be used
    in any place by any caller who receives a reference to it.
    
    Unlike with the Core Reflection API, where access is checked every time
    a reflective method is invoked,
    method handle access checking is performed
    <a href="MethodHandles.Lookup.html#access">when the method handle is created</a>.
    In the case of `ldc` (see below), access checking is performed as part of linking
    the constant pool entry underlying the constant method handle.
    
    Thus, handles to non-public methods, or to methods in non-public classes,
    should generally be kept secret.
    They should not be passed to untrusted code unless their use from
    the untrusted code would be harmless.
    
    <h2>Method handle creation</h2>
    Java code can create a method handle that directly accesses
    any method, constructor, or field that is accessible to that code.
    This is done via a reflective, capability-based API called
    java.lang.invoke.MethodHandles.Lookup MethodHandles.Lookup.
    For example, a static method handle can be obtained
    from java.lang.invoke.MethodHandles.Lookup.findStatic Lookup.findStatic.
    There are also conversion methods from Core Reflection API objects,
    such as java.lang.invoke.MethodHandles.Lookup.unreflect Lookup.unreflect.
    
    Like classes and strings, method handles that correspond to accessible
    fields, methods, and constructors can also be represented directly
    in a class file's constant pool as constants to be loaded by `ldc` bytecodes.
    A new type of constant pool entry, `CONSTANT_MethodHandle`,
    refers directly to an associated `CONSTANT_Methodref`,
    `CONSTANT_InterfaceMethodref`, or `CONSTANT_Fieldref`
    constant pool entry.
    (For full details on method handle constants, see sections 4.4.8 and 5.4.3.5 of the Java Virtual Machine
    Specification.)
    
    Method handles produced by lookups or constant loads from methods or
    constructors with the variable arity modifier bit (`0x0080`)
    have a corresponding variable arity, as if they were defined with
    the help of .asVarargsCollector asVarargsCollector
    or .withVarargs withVarargs.
    
    A method reference may refer either to a static or non-static method.
    In the non-static case, the method handle type includes an explicit
    receiver argument, prepended before any other arguments.
    In the method handle's type, the initial receiver argument is typed
    according to the class under which the method was initially requested.
    (E.g., if a non-static method handle is obtained via `ldc`,
    the type of the receiver is the class named in the constant pool entry.)
    
    Method handle constants are subject to the same link-time access checks
    their corresponding bytecode instructions, and the `ldc` instruction
    will throw corresponding linkage errors if the bytecode behaviors would
    throw such errors.
    
    As a corollary of this, access to protected members is restricted
    to receivers only of the accessing class, or one of its subclasses,
    and the accessing class must in turn be a subclass (or package sibling)
    of the protected member's defining class.
    If a method reference refers to a protected non-static method or field
    of a class outside the current package, the receiver argument will
    be narrowed to the type of the accessing class.
    
    When a method handle to a virtual method is invoked, the method is
    always looked up in the receiver (that is, the first argument).
    
    A non-virtual method handle to a specific virtual method implementation
    can also be created.  These do not perform virtual lookup based on
    receiver type.  Such a method handle simulates the effect of
    an `invokespecial` instruction to the same method.
    A non-virtual method handle can also be created to simulate the effect
    of an `invokevirtual` or `invokeinterface` instruction on
    a private method (as applicable).
    
    <h2>Usage examples</h2>
    Here are some examples of usage:
    <blockquote>````Object x, y; String s; int i;
    MethodType mt; MethodHandle mh;
    MethodHandles.Lookup lookup = MethodHandles.lookup();
    // mt is (char,char)String
    mt = MethodType.methodType(String.class, char.class, char.class);
    mh = lookup.findVirtual(String.class, "replace", mt);
    s = (String) mh.invokeExact("daddy",'d','n');
    // invokeExact(Ljava/lang/String;CC)Ljava/lang/String;
    assertEquals(s, "nanny");
    // weakly typed invocation (using MHs.invoke)
    s = (String) mh.invokeWithArguments("sappy", 'p', 'v');
    assertEquals(s, "savvy");
    // mt is (Object[])List
    mt = MethodType.methodType(java.util.List.class, Object[].class);
    mh = lookup.findStatic(java.util.Arrays.class, "asList", mt);
    assert(mh.isVarargsCollector());
    x = mh.invoke("one", "two");
    // invoke(Ljava/lang/String;Ljava/lang/String;)Ljava/lang/Object;
    assertEquals(x, java.util.Arrays.asList("one","two"));
    // mt is (Object,Object,Object)Object
    mt = MethodType.genericMethodType(3);
    mh = mh.asType(mt);
    x = mh.invokeExact((Object)1, (Object)2, (Object)3);
    // invokeExact(Ljava/lang/Object;Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;
    assertEquals(x, java.util.Arrays.asList(1,2,3));
    // mt is ()int
    mt = MethodType.methodType(int.class);
    mh = lookup.findVirtual(java.util.List.class, "size", mt);
    i = (int) mh.invokeExact(java.util.Arrays.asList(1,2,3));
    // invokeExact(Ljava/util/List;)I
    assert(i == 3);
    mt = MethodType.methodType(void.class, String.class);
    mh = lookup.findVirtual(java.io.PrintStream.class, "println", mt);
    mh.invokeExact(System.out, "Hello, world.");
    // invokeExact(Ljava/io/PrintStream;Ljava/lang/String;)V````</blockquote>
    Each of the above calls to `invokeExact` or plain `invoke`
    generates a single invokevirtual instruction with
    the symbolic type descriptor indicated in the following comment.
    In these examples, the helper method `assertEquals` is assumed to
    be a method which calls java.util.Objects.equals(Object,Object) Objects.equals
    on its arguments, and asserts that the result is True.
    
    <h2>Exceptions</h2>
    The methods `invokeExact` and `invoke` are declared
    to throw java.lang.Throwable Throwable,
    which is to say that there is no static restriction on what a method handle
    can throw.  Since the JVM does not distinguish between checked
    and unchecked exceptions (other than by their class, of course),
    there is no particular effect on bytecode shape from ascribing
    checked exceptions to method handle invocations.  But in Java source
    code, methods which perform method handle calls must either explicitly
    throw `Throwable`, or else must catch all
    throwables locally, rethrowing only those which are legal in the context,
    and wrapping ones which are illegal.
    
    <h2><a id="sigpoly"></a>Signature polymorphism</h2>
    The unusual compilation and linkage behavior of
    `invokeExact` and plain `invoke`
    is referenced by the term *signature polymorphism*.
    As defined in the Java Language Specification,
    a signature polymorphic method is one which can operate with
    any of a wide range of call signatures and return types.
    
    In source code, a call to a signature polymorphic method will
    compile, regardless of the requested symbolic type descriptor.
    As usual, the Java compiler emits an `invokevirtual`
    instruction with the given symbolic type descriptor against the named method.
    The unusual part is that the symbolic type descriptor is derived from
    the actual argument and return types, not from the method declaration.
    
    When the JVM processes bytecode containing signature polymorphic calls,
    it will successfully link any such call, regardless of its symbolic type descriptor.
    (In order to retain type safety, the JVM will guard such calls with suitable
    dynamic type checks, as described elsewhere.)
    
    Bytecode generators, including the compiler back end, are required to emit
    untransformed symbolic type descriptors for these methods.
    Tools which determine symbolic linkage are required to accept such
    untransformed descriptors, without reporting linkage errors.
    
    <h2>Interoperation between method handles and the Core Reflection API</h2>
    Using factory methods in the java.lang.invoke.MethodHandles.Lookup Lookup API,
    any class member represented by a Core Reflection API object
    can be converted to a behaviorally equivalent method handle.
    For example, a reflective java.lang.reflect.Method Method can
    be converted to a method handle using
    java.lang.invoke.MethodHandles.Lookup.unreflect Lookup.unreflect.
    The resulting method handles generally provide more direct and efficient
    access to the underlying class members.
    
    As a special case,
    when the Core Reflection API is used to view the signature polymorphic
    methods `invokeExact` or plain `invoke` in this class,
    they appear as ordinary non-polymorphic methods.
    Their reflective appearance, as viewed by
    java.lang.Class.getDeclaredMethod Class.getDeclaredMethod,
    is unaffected by their special status in this API.
    For example, java.lang.reflect.Method.getModifiers Method.getModifiers
    will report exactly those modifier bits required for any similarly
    declared method, including in this case `native` and `varargs` bits.
    
    As with any reflected method, these methods (when reflected) may be
    invoked via java.lang.reflect.Method.invoke java.lang.reflect.Method.invoke.
    However, such reflective calls do not result in method handle invocations.
    Such a call, if passed the required argument
    (a single one, of type `Object[]`), will ignore the argument and
    will throw an `UnsupportedOperationException`.
    
    Since `invokevirtual` instructions can natively
    invoke method handles under any symbolic type descriptor, this reflective view conflicts
    with the normal presentation of these methods via bytecodes.
    Thus, these two native methods, when reflectively viewed by
    `Class.getDeclaredMethod`, may be regarded as placeholders only.
    
    In order to obtain an invoker method for a particular type descriptor,
    use java.lang.invoke.MethodHandles.exactInvoker MethodHandles.exactInvoker,
    or java.lang.invoke.MethodHandles.invoker MethodHandles.invoker.
    The java.lang.invoke.MethodHandles.Lookup.findVirtual Lookup.findVirtual
    API is also able to return a method handle
    to call `invokeExact` or plain `invoke`,
    for any specified type descriptor .
    
    <h2>Interoperation between method handles and Java generics</h2>
    A method handle can be obtained on a method, constructor, or field
    which is declared with Java generic types.
    As with the Core Reflection API, the type of the method handle
    will be constructed from the erasure of the source-level type.
    When a method handle is invoked, the types of its arguments
    or the return value cast type may be generic types or type instances.
    If this occurs, the compiler will replace those
    types by their erasures when it constructs the symbolic type descriptor
    for the `invokevirtual` instruction.
    
    Method handles do not represent
    their function-like types in terms of Java parameterized (generic) types,
    because there are three mismatches between function-like types and parameterized
    Java types.
    
    - Method types range over all possible arities,
    from no arguments to up to the  <a href="MethodHandle.html#maxarity">maximum number</a> of allowed arguments.
    Generics are not variadic, and so cannot represent this.
    - Method types can specify arguments of primitive types,
    which Java generic types cannot range over.
    - Higher order functions over method handles (combinators) are
    often generic across a wide range of function types, including
    those of multiple arities.  It is impossible to represent such
    genericity with a Java type parameter.
    
    
    <h2><a id="maxarity"></a>Arity limits</h2>
    The JVM imposes on all methods and constructors of any kind an absolute
    limit of 255 stacked arguments.  This limit can appear more restrictive
    in certain cases:
    
    - A `long` or `double` argument counts (for purposes of arity limits) as two argument slots.
    - A non-static method consumes an extra argument for the object on which the method is called.
    - A constructor consumes an extra argument for the object which is being constructed.
    - Since a method handle&rsquo;s `invoke` method (or other signature-polymorphic method) is non-virtual,
        it consumes an extra argument for the method handle itself, in addition to any non-virtual receiver object.
    
    These limits imply that certain method handles cannot be created, solely because of the JVM limit on stacked arguments.
    For example, if a static JVM method accepts exactly 255 arguments, a method handle cannot be created for it.
    Attempts to create method handles with impossible method types lead to an IllegalArgumentException.
    In particular, a method handle&rsquo;s type must not have an arity of the exact maximum 255.

    Author(s)
    - John Rose, JSR 292 EG

    See
    - MethodHandles

    Since
    - 1.7
    """

    def type(self) -> "MethodType":
        """
        Reports the type of this method handle.
        Every invocation of this method handle via `invokeExact` must exactly match this type.

        Returns
        - the method handle type
        """
        ...


    def invokeExact(self, *args: Tuple["Object", ...]) -> "Object":
        """
        Invokes the method handle, allowing any caller type descriptor, but requiring an exact type match.
        The symbolic type descriptor at the call site of `invokeExact` must
        exactly match this method handle's .type() type.
        No conversions are allowed on arguments or return values.
        
        When this method is observed via the Core Reflection API,
        it will appear as a single native method, taking an object array and returning an object.
        If this native method is invoked directly via
        java.lang.reflect.Method.invoke java.lang.reflect.Method.invoke, via JNI,
        or indirectly via java.lang.invoke.MethodHandles.Lookup.unreflect Lookup.unreflect,
        it will throw an `UnsupportedOperationException`.

        Arguments
        - args: the signature-polymorphic parameter list, statically represented using varargs

        Returns
        - the signature-polymorphic result, statically represented using `Object`

        Raises
        - WrongMethodTypeException: if the target's type is not identical with the caller's symbolic type descriptor
        - Throwable: anything thrown by the underlying method propagates unchanged through the method handle call
        """
        ...


    def invoke(self, *args: Tuple["Object", ...]) -> "Object":
        """
        Invokes the method handle, allowing any caller type descriptor,
        and optionally performing conversions on arguments and return values.
        
        If the call site's symbolic type descriptor exactly matches this method handle's .type() type,
        the call proceeds as if by .invokeExact invokeExact.
        
        Otherwise, the call proceeds as if this method handle were first
        adjusted by calling .asType asType to adjust this method handle
        to the required type, and then the call proceeds as if by
        .invokeExact invokeExact on the adjusted method handle.
        
        There is no guarantee that the `asType` call is actually made.
        If the JVM can predict the results of making the call, it may perform
        adaptations directly on the caller's arguments,
        and call the target method handle according to its own exact type.
        
        The resolved type descriptor at the call site of `invoke` must
        be a valid argument to the receivers `asType` method.
        In particular, the caller must specify the same argument arity
        as the callee's type,
        if the callee is not a .asVarargsCollector variable arity collector.
        
        When this method is observed via the Core Reflection API,
        it will appear as a single native method, taking an object array and returning an object.
        If this native method is invoked directly via
        java.lang.reflect.Method.invoke java.lang.reflect.Method.invoke, via JNI,
        or indirectly via java.lang.invoke.MethodHandles.Lookup.unreflect Lookup.unreflect,
        it will throw an `UnsupportedOperationException`.

        Arguments
        - args: the signature-polymorphic parameter list, statically represented using varargs

        Returns
        - the signature-polymorphic result, statically represented using `Object`

        Raises
        - WrongMethodTypeException: if the target's type cannot be adjusted to the caller's symbolic type descriptor
        - ClassCastException: if the target's type can be adjusted to the caller, but a reference cast fails
        - Throwable: anything thrown by the underlying method propagates unchanged through the method handle call
        """
        ...


    def invokeWithArguments(self, *arguments: Tuple["Object", ...]) -> "Object":
        """
        Performs a variable arity invocation, passing the arguments in the given array
        to the method handle, as if via an inexact .invoke invoke from a call site
        which mentions only the type `Object`, and whose actual argument count is the length
        of the argument array.
        
        Specifically, execution proceeds as if by the following steps,
        although the methods are not guaranteed to be called if the JVM
        can predict their effects.
        
        - Determine the length of the argument array as `N`.
            For a null reference, `N=0`. 
        - Collect the `N` elements of the array as a logical
            argument list, each argument statically typed as an `Object`. 
        - Determine, as `M`, the parameter count of the type of this
            method handle. 
        - Determine the general type `TN` of `N` arguments or
            `M` arguments, if smaller than `N`, as
            `TN=MethodType.genericMethodType(Math.min(N, M))`.
        - If `N` is greater than `M`, perform the following
            checks and actions to shorten the logical argument list: 
            - Check that this method handle has variable arity with a
                MethodType.lastParameterType trailing parameter
                of some array type `A[]`.  If not, fail with a
                `WrongMethodTypeException`. 
            - Collect the trailing elements (there are `N-M+1` of them)
                from the logical argument list into a single array of
                type `A[]`, using `asType` conversions to
                convert each trailing argument to type `A`. 
            - If any of these conversions proves impossible, fail with either
                a `ClassCastException` if any trailing element cannot be
                cast to `A` or a `NullPointerException` if any
                trailing element is `null` and `A` is not a reference
                type. 
            - Replace the logical arguments gathered into the array of
                type `A[]` with the array itself, thus shortening
                the argument list to length `M`. This final argument
                retains the static type `A[]`.
            - Adjust the type `TN` by changing the `N`th
                parameter type from `Object` to `A[]`.
            
        - Force the original target method handle `MH0` to the
            required type, as `MH1 = MH0.asType(TN)`. 
        - Spread the argument list into `N` separate arguments `A0, ...`. 
        - Invoke the type-adjusted method handle on the unpacked arguments:
            MH1.invokeExact(A0, ...). 
        - Take the return value as an `Object` reference. 
        
        
        If the target method handle has variable arity, and the argument list is longer
        than that arity, the excess arguments, starting at the position of the trailing
        array argument, will be gathered (if possible, as if by `asType` conversions)
        into an array of the appropriate type, and invocation will proceed on the
        shortened argument list.
        In this way, *jumbo argument lists* which would spread into more
        than 254 slots can still be processed uniformly.
        
        Unlike the .invoke(Object...) generic invocation mode, which can
        "recycle" an array argument, passing it directly to the target method,
        this invocation mode *always* creates a new array parameter, even
        if the original array passed to `invokeWithArguments` would have
        been acceptable as a direct argument to the target method.
        Even if the number `M` of actual arguments is the arity `N`,
        and the last argument is dynamically a suitable array of type `A[]`,
        it will still be boxed into a new one-element array, since the call
        site statically types the argument as `Object`, not an array type.
        This is not a special rule for this method, but rather a regular effect
        of the .asVarargsCollector rules for variable-arity invocation.
        
        Because of the action of the `asType` step, the following argument
        conversions are applied as necessary:
        
        - reference casting
        - unboxing
        - widening primitive conversions
        - variable arity conversion
        
        
        The result returned by the call is boxed if it is a primitive,
        or forced to null if the return type is void.
        
        Unlike the signature polymorphic methods `invokeExact` and `invoke`,
        `invokeWithArguments` can be accessed normally via the Core Reflection API and JNI.
        It can therefore be used as a bridge between native or reflective code and method handles.

        Arguments
        - arguments: the arguments to pass to the target

        Returns
        - the result returned by the target

        Raises
        - ClassCastException: if an argument cannot be converted by reference casting
        - WrongMethodTypeException: if the target's type cannot be adjusted to take the given number of `Object` arguments
        - Throwable: anything thrown by the target method invocation

        See
        - MethodHandles.spreadInvoker

        Unknown Tags
        - This call is approximately equivalent to the following code:
        <blockquote>````// for jumbo argument lists, adapt varargs explicitly:
        int N = (arguments == null? 0: arguments.length);
        int M = this.type.parameterCount();
        int MAX_SAFE = 127;  // 127 longs require 254 slots, which is OK
        if (N > MAX_SAFE && N > M && this.isVarargsCollector()) {
          Class<?> arrayType = this.type().lastParameterType();
          Class<?> elemType = arrayType.getComponentType();
          if (elemType != null) {
            Object args2 = Array.newInstance(elemType, M);
            MethodHandle arraySetter = MethodHandles.arrayElementSetter(arrayType);
            for (int i = 0; i < M; i++) {
              arraySetter.invoke(args2, i, arguments[M-1 + i]);`
            arguments = Arrays.copyOf(arguments, M);
            arguments[M-1] = args2;
            return this.asFixedArity().invokeWithArguments(arguments);
          }
        } // done with explicit varargs processing
        
        // Handle fixed arity and non-jumbo variable arity invocation.
        MethodHandle invoker = MethodHandles.spreadInvoker(this.type(), 0);
        Object result = invoker.invokeExact(this, arguments);
        }```</blockquote>
        """
        ...


    def invokeWithArguments(self, arguments: "java.util.List"[Any]) -> "Object":
        """
        Performs a variable arity invocation, passing the arguments in the given list
        to the method handle, as if via an inexact .invoke invoke from a call site
        which mentions only the type `Object`, and whose actual argument count is the length
        of the argument list.
        
        This method is also equivalent to the following code:
        <blockquote>````invokeWithArguments(arguments.toArray())````</blockquote>
        
        Jumbo-sized lists are acceptable if this method handle has variable arity.
        See .invokeWithArguments(Object[]) for details.

        Arguments
        - arguments: the arguments to pass to the target

        Returns
        - the result returned by the target

        Raises
        - NullPointerException: if `arguments` is a null reference
        - ClassCastException: if an argument cannot be converted by reference casting
        - WrongMethodTypeException: if the target's type cannot be adjusted to take the given number of `Object` arguments
        - Throwable: anything thrown by the target method invocation
        """
        ...


    def asType(self, newType: "MethodType") -> "MethodHandle":
        """
        Produces an adapter method handle which adapts the type of the
        current method handle to a new type.
        The resulting method handle is guaranteed to report a type
        which is equal to the desired new type.
        
        If the original type and new type are equal, returns `this`.
        
        The new method handle, when invoked, will perform the following
        steps:
        
        - Convert the incoming argument list to match the original
            method handle's argument list.
        - Invoke the original method handle on the converted argument list.
        - Convert any result returned by the original method handle
            to the return type of new method handle.
        
        
        This method provides the crucial behavioral difference between
        .invokeExact invokeExact and plain, inexact .invoke invoke.
        The two methods
        perform the same steps when the caller's type descriptor exactly matches
        the callee's, but when the types differ, plain .invoke invoke
        also calls `asType` (or some internal equivalent) in order
        to match up the caller's and callee's types.
        
        If the current method is a variable arity method handle
        argument list conversion may involve the conversion and collection
        of several arguments into an array, as
        .asVarargsCollector described elsewhere.
        In every other case, all conversions are applied *pairwise*,
        which means that each argument or return value is converted to
        exactly one argument or return value (or no return value).
        The applied conversions are defined by consulting
        the corresponding component types of the old and new
        method handle types.
        
        Let *T0* and *T1* be corresponding new and old parameter types,
        or old and new return types.  Specifically, for some valid index `i`, let
        *T0*`=newType.parameterType(i)` and *T1*`=this.type().parameterType(i)`.
        Or else, going the other way for return values, let
        *T0*`=this.type().returnType()` and *T1*`=newType.returnType()`.
        If the types are the same, the new method handle makes no change
        to the corresponding argument or return value (if any).
        Otherwise, one of the following conversions is applied
        if possible:
        
        - If *T0* and *T1* are references, then a cast to *T1* is applied.
            (The types do not need to be related in any particular way.
            This is because a dynamic value of null can convert to any reference type.)
        - If *T0* and *T1* are primitives, then a Java method invocation
            conversion (JLS 5.3) is applied, if one exists.
            (Specifically, *T0* must convert to *T1* by a widening primitive conversion.)
        - If *T0* is a primitive and *T1* a reference,
            a Java casting conversion (JLS 5.5) is applied if one exists.
            (Specifically, the value is boxed from *T0* to its wrapper class,
            which is then widened as needed to *T1*.)
        - If *T0* is a reference and *T1* a primitive, an unboxing
            conversion will be applied at runtime, possibly followed
            by a Java method invocation conversion (JLS 5.3)
            on the primitive value.  (These are the primitive widening conversions.)
            *T0* must be a wrapper class or a supertype of one.
            (In the case where *T0* is Object, these are the conversions
            allowed by java.lang.reflect.Method.invoke java.lang.reflect.Method.invoke.)
            The unboxing conversion must have a possibility of success, which means that
            if *T0* is not itself a wrapper class, there must exist at least one
            wrapper class *TW* which is a subtype of *T0* and whose unboxed
            primitive value can be widened to *T1*.
        - If the return type *T1* is marked as void, any returned value is discarded
        - If the return type *T0* is void and *T1* a reference, a null value is introduced.
        - If the return type *T0* is void and *T1* a primitive,
            a zero value is introduced.
        
        (*Note:* Both *T0* and *T1* may be regarded as static types,
        because neither corresponds specifically to the *dynamic type* of any
        actual argument or return value.)
        
        The method handle conversion cannot be made if any one of the required
        pairwise conversions cannot be made.
        
        At runtime, the conversions applied to reference arguments
        or return values may require additional runtime checks which can fail.
        An unboxing operation may fail because the original reference is null,
        causing a java.lang.NullPointerException NullPointerException.
        An unboxing operation or a reference cast may also fail on a reference
        to an object of the wrong type,
        causing a java.lang.ClassCastException ClassCastException.
        Although an unboxing operation may accept several kinds of wrappers,
        if none are available, a `ClassCastException` will be thrown.

        Arguments
        - newType: the expected type of the new method handle

        Returns
        - a method handle which delegates to `this` after performing
                  any necessary argument conversions, and arranges for any
                  necessary return value conversions

        Raises
        - NullPointerException: if `newType` is a null reference
        - WrongMethodTypeException: if the conversion cannot be made

        See
        - MethodHandles.explicitCastArguments
        """
        ...


    def asSpreader(self, arrayType: type[Any], arrayLength: int) -> "MethodHandle":
        """
        Makes an *array-spreading* method handle, which accepts a trailing array argument
        and spreads its elements as positional arguments.
        The new method handle adapts, as its *target*,
        the current method handle.  The type of the adapter will be
        the same as the type of the target, except that the final
        `arrayLength` parameters of the target's type are replaced
        by a single array parameter of type `arrayType`.
        
        If the array element type differs from any of the corresponding
        argument types on the original target,
        the original target is adapted to take the array elements directly,
        as if by a call to .asType asType.
        
        When called, the adapter replaces a trailing array argument
        by the array's elements, each as its own argument to the target.
        (The order of the arguments is preserved.)
        They are converted pairwise by casting and/or unboxing
        to the types of the trailing parameters of the target.
        Finally the target is called.
        What the target eventually returns is returned unchanged by the adapter.
        
        Before calling the target, the adapter verifies that the array
        contains exactly enough elements to provide a correct argument count
        to the target method handle.
        (The array may also be null when zero elements are required.)
        
        When the adapter is called, the length of the supplied `array`
        argument is queried as if by `array.length` or `arraylength`
        bytecode. If the adapter accepts a zero-length trailing array argument,
        the supplied `array` argument can either be a zero-length array or
        `null`; otherwise, the adapter will throw a `NullPointerException`
        if the array is `null` and throw an IllegalArgumentException
        if the array does not have the correct number of elements.
        
        Here are some simple examples of array-spreading method handles:
        <blockquote>````MethodHandle equals = publicLookup()
          .findVirtual(String.class, "equals", methodType(boolean.class, Object.class));
        assert( (boolean) equals.invokeExact("me", (Object)"me"));
        assert(!(boolean) equals.invokeExact("me", (Object)"thee"));
        // spread both arguments from a 2-array:
        MethodHandle eq2 = equals.asSpreader(Object[].class, 2);
        assert( (boolean) eq2.invokeExact(new Object[]{ "me", "me"`));
        assert(!(boolean) eq2.invokeExact(new Object[]{ "me", "thee" }));
        // try to spread from anything but a 2-array:
        for (int n = 0; n <= 10; n++) {
          Object[] badArityArgs = (n == 2 ? new Object[0] : new Object[n]);
          try { assert((boolean) eq2.invokeExact(badArityArgs) && False); }
          catch (IllegalArgumentException ex) { } // OK
        }
        // spread both arguments from a String array:
        MethodHandle eq2s = equals.asSpreader(String[].class, 2);
        assert( (boolean) eq2s.invokeExact(new String[]{ "me", "me" }));
        assert(!(boolean) eq2s.invokeExact(new String[]{ "me", "thee" }));
        // spread second arguments from a 1-array:
        MethodHandle eq1 = equals.asSpreader(Object[].class, 1);
        assert( (boolean) eq1.invokeExact("me", new Object[]{ "me" }));
        assert(!(boolean) eq1.invokeExact("me", new Object[]{ "thee" }));
        // spread no arguments from a 0-array or null:
        MethodHandle eq0 = equals.asSpreader(Object[].class, 0);
        assert( (boolean) eq0.invokeExact("me", (Object)"me", new Object[0]));
        assert(!(boolean) eq0.invokeExact("me", (Object)"thee", (Object[])null));
        // asSpreader and asCollector are approximate inverses:
        for (int n = 0; n <= 2; n++) {
            for (Class<?> a : new Class<?>[]{Object[].class, String[].class, CharSequence[].class}) {
                MethodHandle equals2 = equals.asSpreader(a, n).asCollector(a, n);
                assert( (boolean) equals2.invokeWithArguments("me", "me"));
                assert(!(boolean) equals2.invokeWithArguments("me", "thee"));
            }
        }
        MethodHandle caToString = publicLookup()
          .findStatic(Arrays.class, "toString", methodType(String.class, char[].class));
        assertEquals("[A, B, C]", (String) caToString.invokeExact("ABC".toCharArray()));
        MethodHandle caString3 = caToString.asCollector(char[].class, 3);
        assertEquals("[A, B, C]", (String) caString3.invokeExact('A', 'B', 'C'));
        MethodHandle caToString2 = caString3.asSpreader(char[].class, 2);
        assertEquals("[A, B, C]", (String) caToString2.invokeExact('A', "BC".toCharArray()));
        }```</blockquote>

        Arguments
        - arrayType: usually `Object[]`, the type of the array argument from which to extract the spread arguments
        - arrayLength: the number of arguments to spread from an incoming array argument

        Returns
        - a new method handle which spreads its final array argument,
                before calling the original method handle

        Raises
        - NullPointerException: if `arrayType` is a null reference
        - IllegalArgumentException: if `arrayType` is not an array type,
                or if target does not have at least
                `arrayLength` parameter types,
                or if `arrayLength` is negative,
                or if the resulting method handle's type would have
                <a href="MethodHandle.html#maxarity">too many parameters</a>
        - WrongMethodTypeException: if the implied `asType` call fails

        See
        - .asCollector
        """
        ...


    def asSpreader(self, spreadArgPos: int, arrayType: type[Any], arrayLength: int) -> "MethodHandle":
        """
        Makes an *array-spreading* method handle, which accepts an array argument at a given position and spreads
        its elements as positional arguments in place of the array. The new method handle adapts, as its *target*,
        the current method handle. The type of the adapter will be the same as the type of the target, except that the
        `arrayLength` parameters of the target's type, starting at the zero-based position `spreadArgPos`,
        are replaced by a single array parameter of type `arrayType`.
        
        This method behaves very much like .asSpreader(Class, int), but accepts an additional `spreadArgPos`
        argument to indicate at which position in the parameter list the spreading should take place.

        Arguments
        - spreadArgPos: the position (zero-based index) in the argument list at which spreading should start.
        - arrayType: usually `Object[]`, the type of the array argument from which to extract the spread arguments
        - arrayLength: the number of arguments to spread from an incoming array argument

        Returns
        - a new method handle which spreads an array argument at a given position,
                before calling the original method handle

        Raises
        - NullPointerException: if `arrayType` is a null reference
        - IllegalArgumentException: if `arrayType` is not an array type,
                or if target does not have at least
                `arrayLength` parameter types,
                or if `arrayLength` is negative,
                or if `spreadArgPos` has an illegal value (negative, or together with arrayLength exceeding the
                number of arguments),
                or if the resulting method handle's type would have
                <a href="MethodHandle.html#maxarity">too many parameters</a>
        - WrongMethodTypeException: if the implied `asType` call fails

        See
        - .asSpreader(Class, int)

        Since
        - 9

        Unknown Tags
        - Example:
        <blockquote>````MethodHandle compare = LOOKUP.findStatic(Objects.class, "compare", methodType(int.class, Object.class, Object.class, Comparator.class));
            MethodHandle compare2FromArray = compare.asSpreader(0, Object[].class, 2);
            Object[] ints = new Object[]{3, 9, 7, 7`;
            Comparator<Integer> cmp = (a, b) -> a - b;
            assertTrue((int) compare2FromArray.invoke(Arrays.copyOfRange(ints, 0, 2), cmp) < 0);
            assertTrue((int) compare2FromArray.invoke(Arrays.copyOfRange(ints, 1, 3), cmp) > 0);
            assertTrue((int) compare2FromArray.invoke(Arrays.copyOfRange(ints, 2, 4), cmp) == 0);
        }```</blockquote>
        """
        ...


    def withVarargs(self, makeVarargs: bool) -> "MethodHandle":
        """
        Adapts this method handle to be .asVarargsCollector variable arity
        if the boolean flag is True, else .asFixedArity fixed arity.
        If the method handle is already of the proper arity mode, it is returned
        unchanged.

        Arguments
        - makeVarargs: True if the return method handle should have variable arity behavior

        Returns
        - a method handle of the same type, with possibly adjusted variable arity behavior

        Raises
        - IllegalArgumentException: if `makeVarargs` is True and
                this method handle does not have a trailing array parameter

        See
        - .asFixedArity

        Since
        - 9

        Unknown Tags
        - This method is sometimes useful when adapting a method handle that
        may be variable arity, to ensure that the resulting adapter is also
        variable arity if and only if the original handle was.  For example,
        this code changes the first argument of a handle `mh` to `int` without
        disturbing its variable arity property:
        `mh.asType(mh.type().changeParameterType(0,int.class))
            .withVarargs(mh.isVarargsCollector())`
        
        This call is approximately equivalent to the following code:
        <blockquote>````if (makeVarargs == isVarargsCollector())
          return this;
        else if (makeVarargs)
          return asVarargsCollector(type().lastParameterType());
        else
          return asFixedArity();````</blockquote>
        """
        ...


    def asCollector(self, arrayType: type[Any], arrayLength: int) -> "MethodHandle":
        """
        Makes an *array-collecting* method handle, which accepts a given number of trailing
        positional arguments and collects them into an array argument.
        The new method handle adapts, as its *target*,
        the current method handle.  The type of the adapter will be
        the same as the type of the target, except that a single trailing
        parameter (usually of type `arrayType`) is replaced by
        `arrayLength` parameters whose type is element type of `arrayType`.
        
        If the array type differs from the final argument type on the original target,
        the original target is adapted to take the array type directly,
        as if by a call to .asType asType.
        
        When called, the adapter replaces its trailing `arrayLength`
        arguments by a single new array of type `arrayType`, whose elements
        comprise (in order) the replaced arguments.
        Finally the target is called.
        What the target eventually returns is returned unchanged by the adapter.
        
        (The array may also be a shared constant when `arrayLength` is zero.)
        
        (*Note:* The `arrayType` is often identical to the
        MethodType.lastParameterType last parameter type
        of the original target.
        It is an explicit argument for symmetry with `asSpreader`, and also
        to allow the target to use a simple `Object` as its last parameter type.)
        
        In order to create a collecting adapter which is not restricted to a particular
        number of collected arguments, use .asVarargsCollector asVarargsCollector
        or .withVarargs withVarargs instead.
        
        Here are some examples of array-collecting method handles:
        <blockquote>````MethodHandle deepToString = publicLookup()
          .findStatic(Arrays.class, "deepToString", methodType(String.class, Object[].class));
        assertEquals("[won]",   (String) deepToString.invokeExact(new Object[]{"won"`));
        MethodHandle ts1 = deepToString.asCollector(Object[].class, 1);
        assertEquals(methodType(String.class, Object.class), ts1.type());
        //assertEquals("[won]", (String) ts1.invokeExact(         new Object[]{"won"})); //FAIL
        assertEquals("[[won]]", (String) ts1.invokeExact((Object) new Object[]{"won"}));
        // arrayType can be a subtype of Object[]
        MethodHandle ts2 = deepToString.asCollector(String[].class, 2);
        assertEquals(methodType(String.class, String.class, String.class), ts2.type());
        assertEquals("[two, too]", (String) ts2.invokeExact("two", "too"));
        MethodHandle ts0 = deepToString.asCollector(Object[].class, 0);
        assertEquals("[]", (String) ts0.invokeExact());
        // collectors can be nested, Lisp-style
        MethodHandle ts22 = deepToString.asCollector(Object[].class, 3).asCollector(String[].class, 2);
        assertEquals("[A, B, [C, D]]", ((String) ts22.invokeExact((Object)'A', (Object)"B", "C", "D")));
        // arrayType can be any primitive array type
        MethodHandle bytesToString = publicLookup()
          .findStatic(Arrays.class, "toString", methodType(String.class, byte[].class))
          .asCollector(byte[].class, 3);
        assertEquals("[1, 2, 3]", (String) bytesToString.invokeExact((byte)1, (byte)2, (byte)3));
        MethodHandle longsToString = publicLookup()
          .findStatic(Arrays.class, "toString", methodType(String.class, long[].class))
          .asCollector(long[].class, 1);
        assertEquals("[123]", (String) longsToString.invokeExact((long)123));
        }```</blockquote>
        
        *Note:* The resulting adapter is never a MethodHandle.asVarargsCollector
        variable-arity method handle, even if the original target method handle was.

        Arguments
        - arrayType: often `Object[]`, the type of the array argument which will collect the arguments
        - arrayLength: the number of arguments to collect into a new array argument

        Returns
        - a new method handle which collects some trailing argument
                into an array, before calling the original method handle

        Raises
        - NullPointerException: if `arrayType` is a null reference
        - IllegalArgumentException: if `arrayType` is not an array type
                or `arrayType` is not assignable to this method handle's trailing parameter type,
                or `arrayLength` is not a legal array size,
                or the resulting method handle's type would have
                <a href="MethodHandle.html#maxarity">too many parameters</a>
        - WrongMethodTypeException: if the implied `asType` call fails

        See
        - .asVarargsCollector
        """
        ...


    def asCollector(self, collectArgPos: int, arrayType: type[Any], arrayLength: int) -> "MethodHandle":
        """
        Makes an *array-collecting* method handle, which accepts a given number of positional arguments starting
        at a given position, and collects them into an array argument. The new method handle adapts, as its
        *target*, the current method handle. The type of the adapter will be the same as the type of the target,
        except that the parameter at the position indicated by `collectArgPos` (usually of type `arrayType`)
        is replaced by `arrayLength` parameters whose type is element type of `arrayType`.
        
        This method behaves very much like .asCollector(Class, int), but differs in that its `collectArgPos` argument indicates at which position in the parameter list arguments should be collected. This
        index is zero-based.

        Arguments
        - collectArgPos: the zero-based position in the parameter list at which to start collecting.
        - arrayType: often `Object[]`, the type of the array argument which will collect the arguments
        - arrayLength: the number of arguments to collect into a new array argument

        Returns
        - a new method handle which collects some arguments
                into an array, before calling the original method handle

        Raises
        - NullPointerException: if `arrayType` is a null reference
        - IllegalArgumentException: if `arrayType` is not an array type
                or `arrayType` is not assignable to this method handle's array parameter type,
                or `arrayLength` is not a legal array size,
                or `collectArgPos` has an illegal value (negative, or greater than the number of arguments),
                or the resulting method handle's type would have
                <a href="MethodHandle.html#maxarity">too many parameters</a>
        - WrongMethodTypeException: if the implied `asType` call fails

        See
        - .asCollector(Class, int)

        Since
        - 9

        Unknown Tags
        - Examples:
        <blockquote>````StringWriter swr = new StringWriter();
            MethodHandle swWrite = LOOKUP.findVirtual(StringWriter.class, "write", methodType(void.class, char[].class, int.class, int.class)).bindTo(swr);
            MethodHandle swWrite4 = swWrite.asCollector(0, char[].class, 4);
            swWrite4.invoke('A', 'B', 'C', 'D', 1, 2);
            assertEquals("BC", swr.toString());
            swWrite4.invoke('P', 'Q', 'R', 'S', 0, 4);
            assertEquals("BCPQRS", swr.toString());
            swWrite4.invoke('W', 'X', 'Y', 'Z', 3, 1);
            assertEquals("BCPQRSZ", swr.toString());````</blockquote>
        
        *Note:* The resulting adapter is never a MethodHandle.asVarargsCollector
        variable-arity method handle, even if the original target method handle was.
        """
        ...


    def asVarargsCollector(self, arrayType: type[Any]) -> "MethodHandle":
        """
        Makes a *variable arity* adapter which is able to accept
        any number of trailing positional arguments and collect them
        into an array argument.
        
        The type and behavior of the adapter will be the same as
        the type and behavior of the target, except that certain
        `invoke` and `asType` requests can lead to
        trailing positional arguments being collected into target's
        trailing parameter.
        Also, the
        MethodType.lastParameterType last parameter type
        of the adapter will be
        `arrayType`, even if the target has a different
        last parameter type.
        
        This transformation may return `this` if the method handle is
        already of variable arity and its trailing parameter type
        is identical to `arrayType`.
        
        When called with .invokeExact invokeExact, the adapter invokes
        the target with no argument changes.
        (*Note:* This behavior is different from a
        .asCollector fixed arity collector,
        since it accepts a whole array of indeterminate length,
        rather than a fixed number of arguments.)
        
        When called with plain, inexact .invoke invoke, if the caller
        type is the same as the adapter, the adapter invokes the target as with
        `invokeExact`.
        (This is the normal behavior for `invoke` when types match.)
        
        Otherwise, if the caller and adapter arity are the same, and the
        trailing parameter type of the caller is a reference type identical to
        or assignable to the trailing parameter type of the adapter,
        the arguments and return values are converted pairwise,
        as if by .asType asType on a fixed arity
        method handle.
        
        Otherwise, the arities differ, or the adapter's trailing parameter
        type is not assignable from the corresponding caller type.
        In this case, the adapter replaces all trailing arguments from
        the original trailing argument position onward, by
        a new array of type `arrayType`, whose elements
        comprise (in order) the replaced arguments.
        
        The caller type must provides as least enough arguments,
        and of the correct type, to satisfy the target's requirement for
        positional arguments before the trailing array argument.
        Thus, the caller must supply, at a minimum, `N-1` arguments,
        where `N` is the arity of the target.
        Also, there must exist conversions from the incoming arguments
        to the target's arguments.
        As with other uses of plain `invoke`, if these basic
        requirements are not fulfilled, a `WrongMethodTypeException`
        may be thrown.
        
        In all cases, what the target eventually returns is returned unchanged by the adapter.
        
        In the final case, it is exactly as if the target method handle were
        temporarily adapted with a .asCollector fixed arity collector
        to the arity required by the caller type.
        (As with `asCollector`, if the array length is zero,
        a shared constant may be used instead of a new array.
        If the implied call to `asCollector` would throw
        an `IllegalArgumentException` or `WrongMethodTypeException`,
        the call to the variable arity adapter must throw
        `WrongMethodTypeException`.)
        
        The behavior of .asType asType is also specialized for
        variable arity adapters, to maintain the invariant that
        plain, inexact `invoke` is always equivalent to an `asType`
        call to adjust the target type, followed by `invokeExact`.
        Therefore, a variable arity adapter responds
        to an `asType` request by building a fixed arity collector,
        if and only if the adapter and requested type differ either
        in arity or trailing argument type.
        The resulting fixed arity collector has its type further adjusted
        (if necessary) to the requested type by pairwise conversion,
        as if by another application of `asType`.
        
        When a method handle is obtained by executing an `ldc` instruction
        of a `CONSTANT_MethodHandle` constant, and the target method is marked
        as a variable arity method (with the modifier bit `0x0080`),
        the method handle will accept multiple arities, as if the method handle
        constant were created by means of a call to `asVarargsCollector`.
        
        In order to create a collecting adapter which collects a predetermined
        number of arguments, and whose type reflects this predetermined number,
        use .asCollector asCollector instead.
        
        No method handle transformations produce new method handles with
        variable arity, unless they are documented as doing so.
        Therefore, besides `asVarargsCollector` and `withVarargs`,
        all methods in `MethodHandle` and `MethodHandles`
        will return a method handle with fixed arity,
        except in the cases where they are specified to return their original
        operand (e.g., `asType` of the method handle's own type).
        
        Calling `asVarargsCollector` on a method handle which is already
        of variable arity will produce a method handle with the same type and behavior.
        It may (or may not) return the original variable arity method handle.
        
        Here is an example, of a list-making variable arity method handle:
        <blockquote>````MethodHandle deepToString = publicLookup()
          .findStatic(Arrays.class, "deepToString", methodType(String.class, Object[].class));
        MethodHandle ts1 = deepToString.asVarargsCollector(Object[].class);
        assertEquals("[won]",   (String) ts1.invokeExact(    new Object[]{"won"`));
        assertEquals("[won]",   (String) ts1.invoke(         new Object[]{"won"}));
        assertEquals("[won]",   (String) ts1.invoke(                      "won" ));
        assertEquals("[[won]]", (String) ts1.invoke((Object) new Object[]{"won"}));
        // findStatic of Arrays.asList(...) produces a variable arity method handle:
        MethodHandle asList = publicLookup()
          .findStatic(Arrays.class, "asList", methodType(List.class, Object[].class));
        assertEquals(methodType(List.class, Object[].class), asList.type());
        assert(asList.isVarargsCollector());
        assertEquals("[]", asList.invoke().toString());
        assertEquals("[1]", asList.invoke(1).toString());
        assertEquals("[two, too]", asList.invoke("two", "too").toString());
        String[] argv = { "three", "thee", "tee" };
        assertEquals("[three, thee, tee]", asList.invoke(argv).toString());
        assertEquals("[three, thee, tee]", asList.invoke((Object[])argv).toString());
        List ls = (List) asList.invoke((Object)argv);
        assertEquals(1, ls.size());
        assertEquals("[three, thee, tee]", Arrays.toString((Object[])ls.get(0)));
        }```</blockquote>
        <p style="font-size:smaller;">
        *Discussion:*
        These rules are designed as a dynamically-typed variation
        of the Java rules for variable arity methods.
        In both cases, callers to a variable arity method or method handle
        can either pass zero or more positional arguments, or else pass
        pre-collected arrays of any length.  Users should be aware of the
        special role of the final argument, and of the effect of a
        type match on that final argument, which determines whether
        or not a single trailing argument is interpreted as a whole
        array or a single element of an array to be collected.
        Note that the dynamic type of the trailing argument has no
        effect on this decision, only a comparison between the symbolic
        type descriptor of the call site and the type descriptor of the method handle.)

        Arguments
        - arrayType: often `Object[]`, the type of the array argument which will collect the arguments

        Returns
        - a new method handle which can collect any number of trailing arguments
                into an array, before calling the original method handle

        Raises
        - NullPointerException: if `arrayType` is a null reference
        - IllegalArgumentException: if `arrayType` is not an array type
                or `arrayType` is not assignable to this method handle's trailing parameter type

        See
        - .asFixedArity
        """
        ...


    def isVarargsCollector(self) -> bool:
        """
        Determines if this method handle
        supports .asVarargsCollector variable arity calls.
        Such method handles arise from the following sources:
        
        - a call to .asVarargsCollector asVarargsCollector
        - a call to a java.lang.invoke.MethodHandles.Lookup lookup method
            which resolves to a variable arity Java method or constructor
        - an `ldc` instruction of a `CONSTANT_MethodHandle`
            which resolves to a variable arity Java method or constructor

        Returns
        - True if this method handle accepts more than one arity of plain, inexact `invoke` calls

        See
        - .asFixedArity
        """
        ...


    def asFixedArity(self) -> "MethodHandle":
        """
        Makes a *fixed arity* method handle which is otherwise
        equivalent to the current method handle.
        
        If the current method handle is not of
        .asVarargsCollector variable arity,
        the current method handle is returned.
        This is True even if the current method handle
        could not be a valid input to `asVarargsCollector`.
        
        Otherwise, the resulting fixed-arity method handle has the same
        type and behavior of the current method handle,
        except that .isVarargsCollector isVarargsCollector
        will be False.
        The fixed-arity method handle may (or may not) be the
        a previous argument to `asVarargsCollector`.
        
        Here is an example, of a list-making variable arity method handle:
        <blockquote>````MethodHandle asListVar = publicLookup()
          .findStatic(Arrays.class, "asList", methodType(List.class, Object[].class))
          .asVarargsCollector(Object[].class);
        MethodHandle asListFix = asListVar.asFixedArity();
        assertEquals("[1]", asListVar.invoke(1).toString());
        Exception caught = null;
        try { asListFix.invoke((Object)1);`
        catch (Exception ex) { caught = ex; }
        assert(caught instanceof ClassCastException);
        assertEquals("[two, too]", asListVar.invoke("two", "too").toString());
        try { asListFix.invoke("two", "too"); }
        catch (Exception ex) { caught = ex; }
        assert(caught instanceof WrongMethodTypeException);
        Object[] argv = { "three", "thee", "tee" };
        assertEquals("[three, thee, tee]", asListVar.invoke(argv).toString());
        assertEquals("[three, thee, tee]", asListFix.invoke(argv).toString());
        assertEquals(1, ((List) asListVar.invoke((Object)argv)).size());
        assertEquals("[three, thee, tee]", asListFix.invoke((Object)argv).toString());
        }```</blockquote>

        Returns
        - a new method handle which accepts only a fixed number of arguments

        See
        - .withVarargs
        """
        ...


    def bindTo(self, x: "Object") -> "MethodHandle":
        """
        Binds a value `x` to the first argument of a method handle, without invoking it.
        The new method handle adapts, as its *target*,
        the current method handle by binding it to the given argument.
        The type of the bound handle will be
        the same as the type of the target, except that a single leading
        reference parameter will be omitted.
        
        When called, the bound handle inserts the given value `x`
        as a new leading argument to the target.  The other arguments are
        also passed unchanged.
        What the target eventually returns is returned unchanged by the bound handle.
        
        The reference `x` must be convertible to the first parameter
        type of the target.
        
        *Note:*  Because method handles are immutable, the target method handle
        retains its original type and behavior.
        
        *Note:* The resulting adapter is never a MethodHandle.asVarargsCollector
        variable-arity method handle, even if the original target method handle was.

        Arguments
        - x: the value to bind to the first argument of the target

        Returns
        - a new method handle which prepends the given value to the incoming
                argument list, before calling the original method handle

        Raises
        - IllegalArgumentException: if the target does not have a
                leading parameter type that is a reference type
        - ClassCastException: if `x` cannot be converted
                to the leading parameter type of the target

        See
        - MethodHandles.insertArguments
        """
        ...


    def describeConstable(self) -> "Optional"["MethodHandleDesc"]:
        """
        Return a nominal descriptor for this instance, if one can be
        constructed, or an empty Optional if one cannot be.

        Returns
        - An Optional containing the resulting nominal descriptor,
        or an empty Optional if one cannot be constructed.

        Since
        - 12
        """
        ...


    def toString(self) -> str:
        """
        Returns a string representation of the method handle,
        starting with the string `"MethodHandle"` and
        ending with the string representation of the method handle's type.
        In other words, this method returns a string equal to the value of:
        <blockquote>````"MethodHandle" + type().toString()````</blockquote>
        
        (*Note:*  Future releases of this API may add further information
        to the string representation.
        Therefore, the present syntax should not be parsed by applications.)

        Returns
        - a string representation of the method handle
        """
        ...



"""
Python module generated from Java source file java.nio.file.Path

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import File
from java.io import IOException
from java.net import URI
from java.nio.file import *
from java.nio.file.spi import FileSystemProvider
from java.util import Iterator
from java.util import NoSuchElementException
from typing import Any, Callable, Iterable, Tuple


class Path(Comparable, Iterable, Watchable):

    @staticmethod
    def of(first: str, *more: Tuple[str, ...]) -> "Path":
        """
        Returns a `Path` by converting a path string, or a sequence of
        strings that when joined form a path string. If `more` does not
        specify any elements then the value of the `first` parameter is
        the path string to convert. If `more` specifies one or more
        elements then each non-empty string, including `first`, is
        considered to be a sequence of name elements and is joined to form a
        path string. The details as to how the Strings are joined is provider
        specific but typically they will be joined using the
        FileSystem.getSeparator name-separator as the separator.
        For example, if the name separator is "`/`" and
        `getPath("/foo","bar","gus")` is invoked, then the path string
        `"/foo/bar/gus"` is converted to a `Path`. A `Path`
        representing an empty path is returned if `first` is the empty
        string and `more` does not contain any non-empty strings.
        
         The `Path` is obtained by invoking the FileSystem.getPath
        getPath method of the FileSystems.getDefault default FileSystem.
        
         Note that while this method is very convenient, using it will imply
        an assumed reference to the default `FileSystem` and limit the
        utility of the calling code. Hence it should not be used in library code
        intended for flexible reuse. A more flexible alternative is to use an
        existing `Path` instance as an anchor, such as:
        ````Path dir = ...
            Path path = dir.resolve("file");````

        Arguments
        - first: the path string or initial part of the path string
        - more: additional strings to be joined to form the path string

        Returns
        - the resulting `Path`

        Raises
        - InvalidPathException: if the path string cannot be converted to a `Path`

        See
        - FileSystem.getPath

        Since
        - 11
        """
        ...


    @staticmethod
    def of(uri: "URI") -> "Path":
        """
        Returns a `Path` by converting a URI.
        
         This method iterates over the FileSystemProvider.installedProviders()
        installed providers to locate the provider that is identified by the
        URI URI.getScheme scheme of the given URI. URI schemes are
        compared without regard to case. If the provider is found then its FileSystemProvider.getPath getPath method is invoked to convert the
        URI.
        
         In the case of the default provider, identified by the URI scheme
        "file", the given URI has a non-empty path component, and undefined query
        and fragment components. Whether the authority component may be present
        is platform specific. The returned `Path` is associated with the
        FileSystems.getDefault default file system.
        
         The default provider provides a similar *round-trip* guarantee
        to the java.io.File class. For a given `Path` *p* it
        is guaranteed that
        <blockquote>`Path.of(`*p*`.`Path.toUri() toUri`()).equals(`
        *p*`.`Path.toAbsolutePath() toAbsolutePath`())`
        </blockquote>
        so long as the original `Path`, the `URI`, and the new `Path` are all created in (possibly different invocations of) the same
        Java virtual machine. Whether other providers make any guarantees is
        provider specific and therefore unspecified.

        Arguments
        - uri: the URI to convert

        Returns
        - the resulting `Path`

        Raises
        - IllegalArgumentException: if preconditions on the `uri` parameter do not hold. The
                 format of the URI is provider specific.
        - FileSystemNotFoundException: The file system, identified by the URI, does not exist and
                 cannot be created automatically, or the provider identified by
                 the URI's scheme component is not installed
        - SecurityException: if a security manager is installed and it denies an unspecified
                 permission to access the file system

        Since
        - 11
        """
        ...


    def getFileSystem(self) -> "FileSystem":
        """
        Returns the file system that created this object.

        Returns
        - the file system that created this object
        """
        ...


    def isAbsolute(self) -> bool:
        """
        Tells whether or not this path is absolute.
        
         An absolute path is complete in that it doesn't need to be combined
        with other path information in order to locate a file.

        Returns
        - `True` if, and only if, this path is absolute
        """
        ...


    def getRoot(self) -> "Path":
        """
        Returns the root component of this path as a `Path` object,
        or `null` if this path does not have a root component.

        Returns
        - a path representing the root component of this path,
                 or `null`
        """
        ...


    def getFileName(self) -> "Path":
        """
        Returns the name of the file or directory denoted by this path as a
        `Path` object. The file name is the *farthest* element from
        the root in the directory hierarchy.

        Returns
        - a path representing the name of the file or directory, or
                 `null` if this path has zero elements
        """
        ...


    def getParent(self) -> "Path":
        """
        Returns the *parent path*, or `null` if this path does not
        have a parent.
        
         The parent of this path object consists of this path's root
        component, if any, and each element in the path except for the
        *farthest* from the root in the directory hierarchy. This method
        does not access the file system; the path or its parent may not exist.
        Furthermore, this method does not eliminate special names such as "."
        and ".." that may be used in some implementations. On UNIX for example,
        the parent of "`/a/b/c`" is "`/a/b`", and the parent of
        `"x/y/.`" is "`x/y`". This method may be used with the .normalize normalize method, to eliminate redundant names, for cases where
        *shell-like* navigation is required.
        
         If this path has more than one element, and no root component, then
        this method is equivalent to evaluating the expression:
        <blockquote>```
        subpath(0,&nbsp;getNameCount()-1);
        ```</blockquote>

        Returns
        - a path representing the path's parent
        """
        ...


    def getNameCount(self) -> int:
        """
        Returns the number of name elements in the path.

        Returns
        - the number of elements in the path, or `0` if this path
                 only represents a root component
        """
        ...


    def getName(self, index: int) -> "Path":
        """
        Returns a name element of this path as a `Path` object.
        
         The `index` parameter is the index of the name element to return.
        The element that is *closest* to the root in the directory hierarchy
        has index `0`. The element that is *farthest* from the root
        has index .getNameCount count`-1`.

        Arguments
        - index: the index of the element

        Returns
        - the name element

        Raises
        - IllegalArgumentException: if `index` is negative, `index` is greater than or
                 equal to the number of elements, or this path has zero name
                 elements
        """
        ...


    def subpath(self, beginIndex: int, endIndex: int) -> "Path":
        """
        Returns a relative `Path` that is a subsequence of the name
        elements of this path.
        
         The `beginIndex` and `endIndex` parameters specify the
        subsequence of name elements. The name that is *closest* to the root
        in the directory hierarchy has index `0`. The name that is
        *farthest* from the root has index .getNameCount
        count`-1`. The returned `Path` object has the name elements
        that begin at `beginIndex` and extend to the element at index `endIndex-1`.

        Arguments
        - beginIndex: the index of the first element, inclusive
        - endIndex: the index of the last element, exclusive

        Returns
        - a new `Path` object that is a subsequence of the name
                 elements in this `Path`

        Raises
        - IllegalArgumentException: if `beginIndex` is negative, or greater than or equal to
                 the number of elements. If `endIndex` is less than or
                 equal to `beginIndex`, or larger than the number of elements.
        """
        ...


    def startsWith(self, other: "Path") -> bool:
        """
        Tests if this path starts with the given path.
        
         This path *starts* with the given path if this path's root
        component *starts* with the root component of the given path,
        and this path starts with the same name elements as the given path.
        If the given path has more name elements than this path then `False`
        is returned.
        
         Whether or not the root component of this path starts with the root
        component of the given path is file system specific. If this path does
        not have a root component and the given path has a root component then
        this path does not start with the given path.
        
         If the given path is associated with a different `FileSystem`
        to this path then `False` is returned.

        Arguments
        - other: the given path

        Returns
        - `True` if this path starts with the given path; otherwise
                 `False`
        """
        ...


    def startsWith(self, other: str) -> bool:
        """
        Tests if this path starts with a `Path`, constructed by converting
        the given path string, in exactly the manner specified by the .startsWith(Path) startsWith(Path) method. On UNIX for example, the path
        "`foo/bar`" starts with "`foo`" and "`foo/bar`". It
        does not start with "`f`" or "`fo`".

        Arguments
        - other: the given path string

        Returns
        - `True` if this path starts with the given path; otherwise
                 `False`

        Raises
        - InvalidPathException: If the path string cannot be converted to a Path.

        Unknown Tags
        - The default implementation is equivalent for this path to:
        ````startsWith(getFileSystem().getPath(other));````
        """
        ...


    def endsWith(self, other: "Path") -> bool:
        """
        Tests if this path ends with the given path.
        
         If the given path has *N* elements, and no root component,
        and this path has *N* or more elements, then this path ends with
        the given path if the last *N* elements of each path, starting at
        the element farthest from the root, are equal.
        
         If the given path has a root component then this path ends with the
        given path if the root component of this path *ends with* the root
        component of the given path, and the corresponding elements of both paths
        are equal. Whether or not the root component of this path ends with the
        root component of the given path is file system specific. If this path
        does not have a root component and the given path has a root component
        then this path does not end with the given path.
        
         If the given path is associated with a different `FileSystem`
        to this path then `False` is returned.

        Arguments
        - other: the given path

        Returns
        - `True` if this path ends with the given path; otherwise
                 `False`
        """
        ...


    def endsWith(self, other: str) -> bool:
        """
        Tests if this path ends with a `Path`, constructed by converting
        the given path string, in exactly the manner specified by the .endsWith(Path) endsWith(Path) method. On UNIX for example, the path
        "`foo/bar`" ends with "`foo/bar`" and "`bar`". It does
        not end with "`r`" or "`/bar`". Note that trailing separators
        are not taken into account, and so invoking this method on the `Path`"`foo/bar`" with the `String` "`bar/`" returns
        `True`.

        Arguments
        - other: the given path string

        Returns
        - `True` if this path ends with the given path; otherwise
                 `False`

        Raises
        - InvalidPathException: If the path string cannot be converted to a Path.

        Unknown Tags
        - The default implementation is equivalent for this path to:
        ````endsWith(getFileSystem().getPath(other));````
        """
        ...


    def normalize(self) -> "Path":
        """
        Returns a path that is this path with redundant name elements eliminated.
        
         The precise definition of this method is implementation dependent but
        in general it derives from this path, a path that does not contain
        *redundant* name elements. In many file systems, the "`.`"
        and "`..`" are special names used to indicate the current directory
        and parent directory. In such file systems all occurrences of "`.`"
        are considered redundant. If a "`..`" is preceded by a
        non-"`..`" name then both names are considered redundant (the
        process to identify such names is repeated until it is no longer
        applicable).
        
         This method does not access the file system; the path may not locate
        a file that exists. Eliminating "`..`" and a preceding name from a
        path may result in the path that locates a different file than the original
        path. This can arise when the preceding name is a symbolic link.

        Returns
        - the resulting path or this path if it does not contain
                 redundant name elements; an empty path is returned if this path
                 does not have a root component and all name elements are redundant

        See
        - .toRealPath
        """
        ...


    def resolve(self, other: "Path") -> "Path":
        """
        Resolve the given path against this path.
        
         If the `other` parameter is an .isAbsolute() absolute
        path then this method trivially returns `other`. If `other`
        is an *empty path* then this method trivially returns this path.
        Otherwise this method considers this path to be a directory and resolves
        the given path against this path. In the simplest case, the given path
        does not have a .getRoot root component, in which case this method
        *joins* the given path to this path and returns a resulting path
        that .endsWith ends with the given path. Where the given path has
        a root component then resolution is highly implementation dependent and
        therefore unspecified.

        Arguments
        - other: the path to resolve against this path

        Returns
        - the resulting path

        See
        - .relativize
        """
        ...


    def resolve(self, other: str) -> "Path":
        """
        Converts a given path string to a `Path` and resolves it against
        this `Path` in exactly the manner specified by the .resolve(Path) resolve method. For example, suppose that the name
        separator is "`/`" and a path represents "`foo/bar`", then
        invoking this method with the path string "`gus`" will result in
        the `Path` "`foo/bar/gus`".

        Arguments
        - other: the path string to resolve against this path

        Returns
        - the resulting path

        Raises
        - InvalidPathException: if the path string cannot be converted to a Path.

        See
        - FileSystem.getPath

        Unknown Tags
        - The default implementation is equivalent for this path to:
        ````resolve(getFileSystem().getPath(other));````
        """
        ...


    def resolveSibling(self, other: "Path") -> "Path":
        """
        Resolves the given path against this path's .getParent parent
        path. This is useful where a file name needs to be *replaced* with
        another file name. For example, suppose that the name separator is
        "`/`" and a path represents "`dir1/dir2/foo`", then invoking
        this method with the `Path` "`bar`" will result in the `Path` "`dir1/dir2/bar`". If this path does not have a parent path,
        or `other` is .isAbsolute() absolute, then this method
        returns `other`. If `other` is an empty path then this method
        returns this path's parent, or where this path doesn't have a parent, the
        empty path.

        Arguments
        - other: the path to resolve against this path's parent

        Returns
        - the resulting path

        See
        - .resolve(Path)

        Unknown Tags
        - The default implementation is equivalent for this path to:
        ````(getParent() == null) ? other : getParent().resolve(other);````
        unless `other == null`, in which case a
        `NullPointerException` is thrown.
        """
        ...


    def resolveSibling(self, other: str) -> "Path":
        """
        Converts a given path string to a `Path` and resolves it against
        this path's .getParent parent path in exactly the manner
        specified by the .resolveSibling(Path) resolveSibling method.

        Arguments
        - other: the path string to resolve against this path's parent

        Returns
        - the resulting path

        Raises
        - InvalidPathException: if the path string cannot be converted to a Path.

        See
        - FileSystem.getPath

        Unknown Tags
        - The default implementation is equivalent for this path to:
        ````resolveSibling(getFileSystem().getPath(other));````
        """
        ...


    def relativize(self, other: "Path") -> "Path":
        """
        Constructs a relative path between this path and a given path.
        
         Relativization is the inverse of .resolve(Path) resolution.
        This method attempts to construct a .isAbsolute relative path
        that when .resolve(Path) resolved against this path, yields a
        path that locates the same file as the given path. For example, on UNIX,
        if this path is `"/a/b"` and the given path is `"/a/b/c/d"`
        then the resulting relative path would be `"c/d"`. Where this
        path and the given path do not have a .getRoot root component,
        then a relative path can be constructed. A relative path cannot be
        constructed if only one of the paths have a root component. Where both
        paths have a root component then it is implementation dependent if a
        relative path can be constructed. If this path and the given path are
        .equals equal then an *empty path* is returned.
        
         For any two .normalize normalized paths *p* and
        *q*, where *q* does not have a root component,
        <blockquote>
          *p*`.relativize(`*p*
          `.resolve(`*q*`)).equals(`*q*`)`
        </blockquote>
        
         When symbolic links are supported, then whether the resulting path,
        when resolved against this path, yields a path that can be used to locate
        the Files.isSameFile same file as `other` is implementation
        dependent. For example, if this path is  `"/a/b"` and the given
        path is `"/a/x"` then the resulting relative path may be `"../x"`. If `"b"` is a symbolic link then is implementation
        dependent if `"a/b/../x"` would locate the same file as `"/a/x"`.

        Arguments
        - other: the path to relativize against this path

        Returns
        - the resulting relative path, or an empty path if both paths are
                 equal

        Raises
        - IllegalArgumentException: if `other` is not a `Path` that can be relativized
                 against this path
        """
        ...


    def toUri(self) -> "URI":
        """
        Returns a URI to represent this path.
        
         This method constructs an absolute URI with a URI.getScheme() scheme equal to the URI scheme that identifies the
        provider. The exact form of the scheme specific part is highly provider
        dependent.
        
         In the case of the default provider, the URI is hierarchical with
        a URI.getPath() path component that is absolute. The query and
        fragment components are undefined. Whether the authority component is
        defined or not is implementation dependent. There is no guarantee that
        the `URI` may be used to construct a java.io.File java.io.File.
        In particular, if this path represents a Universal Naming Convention (UNC)
        path, then the UNC server name may be encoded in the authority component
        of the resulting URI. In the case of the default provider, and the file
        exists, and it can be determined that the file is a directory, then the
        resulting `URI` will end with a slash.
        
         The default provider provides a similar *round-trip* guarantee
        to the java.io.File class. For a given `Path` *p* it
        is guaranteed that
        <blockquote>
        Path.of(URI) Path.of`(`*p*`.toUri()).equals(`*p*
        `.`.toAbsolutePath() toAbsolutePath`())`
        </blockquote>
        so long as the original `Path`, the `URI`, and the new `Path` are all created in (possibly different invocations of) the same
        Java virtual machine. Whether other providers make any guarantees is
        provider specific and therefore unspecified.
        
         When a file system is constructed to access the contents of a file
        as a file system then it is highly implementation specific if the returned
        URI represents the given path in the file system or it represents a
        *compound* URI that encodes the URI of the enclosing file system.
        A format for compound URIs is not defined in this release; such a scheme
        may be added in a future release.

        Returns
        - the URI representing this path

        Raises
        - java.io.IOError: if an I/O error occurs obtaining the absolute path, or where a
                 file system is constructed to access the contents of a file as
                 a file system, and the URI of the enclosing file system cannot be
                 obtained
        - SecurityException: In the case of the default provider, and a security manager
                 is installed, the .toAbsolutePath toAbsolutePath method
                 throws a security exception.
        """
        ...


    def toAbsolutePath(self) -> "Path":
        """
        Returns a `Path` object representing the absolute path of this
        path.
        
         If this path is already Path.isAbsolute absolute then this
        method simply returns this path. Otherwise, this method resolves the path
        in an implementation dependent manner, typically by resolving the path
        against a file system default directory. Depending on the implementation,
        this method may throw an I/O error if the file system is not accessible.

        Returns
        - a `Path` object representing the absolute path

        Raises
        - java.io.IOError: if an I/O error occurs
        - SecurityException: In the case of the default provider, a security manager
                 is installed, and this path is not absolute, then the security
                 manager's SecurityManager.checkPropertyAccess(String)
                 checkPropertyAccess method is invoked to check access to the
                 system property `user.dir`
        """
        ...


    def toRealPath(self, *options: Tuple["LinkOption", ...]) -> "Path":
        """
        Returns the *real* path of an existing file.
        
         The precise definition of this method is implementation dependent but
        in general it derives from this path, an .isAbsolute absolute
        path that locates the Files.isSameFile same file as this path, but
        with name elements that represent the actual name of the directories
        and the file. For example, where filename comparisons on a file system
        are case insensitive then the name elements represent the names in their
        actual case. Additionally, the resulting path has redundant name
        elements removed.
        
         If this path is relative then its absolute path is first obtained,
        as if by invoking the .toAbsolutePath toAbsolutePath method.
        
         The `options` array may be used to indicate how symbolic links
        are handled. By default, symbolic links are resolved to their final
        target. If the option LinkOption.NOFOLLOW_LINKS NOFOLLOW_LINKS is
        present then this method does not resolve symbolic links.
        
        Some implementations allow special names such as "`..`" to refer to
        the parent directory. When deriving the *real path*, and a
        "`..`" (or equivalent) is preceded by a non-"`..`" name then
        an implementation will typically cause both names to be removed. When
        not resolving symbolic links and the preceding name is a symbolic link
        then the names are only removed if it guaranteed that the resulting path
        will locate the same file as this path.

        Arguments
        - options: options indicating how symbolic links are handled

        Returns
        - an absolute path represent the *real* path of the file
                 located by this object

        Raises
        - IOException: if the file does not exist or an I/O error occurs
        - SecurityException: In the case of the default provider, and a security manager
                 is installed, its SecurityManager.checkRead(String) checkRead
                 method is invoked to check read access to the file, and where
                 this path is not absolute, its SecurityManager.checkPropertyAccess(String)
                 checkPropertyAccess method is invoked to check access to the
                 system property `user.dir`
        """
        ...


    def toFile(self) -> "File":
        """
        Returns a File object representing this path. Where this `Path` is associated with the default provider, then this method is
        equivalent to returning a `File` object constructed with the
        `String` representation of this path.
        
         If this path was created by invoking the `File` File.toPath toPath method then there is no guarantee that the `File` object returned by this method is .equals equal to the
        original `File`.

        Returns
        - a `File` object representing this path

        Raises
        - UnsupportedOperationException: if this `Path` is not associated with the default provider

        Unknown Tags
        - The default implementation is equivalent for this path to:
        ````new File(toString());````
        if the `FileSystem` which created this `Path` is the default
        file system; otherwise an `UnsupportedOperationException` is
        thrown.
        """
        ...


    def register(self, watcher: "WatchService", events: list["WatchEvent.Kind"[Any]], *modifiers: Tuple["WatchEvent.Modifier", ...]) -> "WatchKey":
        """
        Registers the file located by this path with a watch service.
        
         In this release, this path locates a directory that exists. The
        directory is registered with the watch service so that entries in the
        directory can be watched. The `events` parameter is the events to
        register and may contain the following events:
        
          - StandardWatchEventKinds.ENTRY_CREATE ENTRY_CREATE -
              entry created or moved into the directory
          - StandardWatchEventKinds.ENTRY_DELETE ENTRY_DELETE -
               entry deleted or moved out of the directory
          - StandardWatchEventKinds.ENTRY_MODIFY ENTRY_MODIFY -
               entry in directory was modified
        
        
         The WatchEvent.context context for these events is the
        relative path between the directory located by this path, and the path
        that locates the directory entry that is created, deleted, or modified.
        
         The set of events may include additional implementation specific
        event that are not defined by the enum StandardWatchEventKinds
        
         The `modifiers` parameter specifies *modifiers* that
        qualify how the directory is registered. This release does not define any
        *standard* modifiers. It may contain implementation specific
        modifiers.
        
         Where a file is registered with a watch service by means of a symbolic
        link then it is implementation specific if the watch continues to depend
        on the existence of the symbolic link after it is registered.

        Arguments
        - watcher: the watch service to which this object is to be registered
        - events: the events for which this object should be registered
        - modifiers: the modifiers, if any, that modify how the object is registered

        Returns
        - a key representing the registration of this object with the
                 given watch service

        Raises
        - UnsupportedOperationException: if unsupported events or modifiers are specified
        - IllegalArgumentException: if an invalid combination of events or modifiers is specified
        - ClosedWatchServiceException: if the watch service is closed
        - NotDirectoryException: if the file is registered to watch the entries in a directory
                 and the file is not a directory  *(optional specific exception)*
        - IOException: if an I/O error occurs
        - SecurityException: In the case of the default provider, and a security manager is
                 installed, the SecurityManager.checkRead(String) checkRead
                 method is invoked to check read access to the file.
        """
        ...


    def register(self, watcher: "WatchService", *events: Tuple["WatchEvent.Kind"[Any], ...]) -> "WatchKey":
        """
        Registers the file located by this path with a watch service.
        
         An invocation of this method behaves in exactly the same way as the
        invocation
        ```
            watchable..register(WatchService,WatchEvent.Kind[],WatchEvent.Modifier[]) register(watcher, events, new WatchEvent.Modifier[0]);
        ```
        
         **Usage Example:**
        Suppose we wish to register a directory for entry create, delete, and modify
        events:
        ```
            Path dir = ...
            WatchService watcher = ...
        
            WatchKey key = dir.register(watcher, ENTRY_CREATE, ENTRY_DELETE, ENTRY_MODIFY);
        ```

        Arguments
        - watcher: The watch service to which this object is to be registered
        - events: The events for which this object should be registered

        Returns
        - A key representing the registration of this object with the
                 given watch service

        Raises
        - UnsupportedOperationException: If unsupported events are specified
        - IllegalArgumentException: If an invalid combination of events is specified
        - ClosedWatchServiceException: If the watch service is closed
        - NotDirectoryException: If the file is registered to watch the entries in a directory
                 and the file is not a directory  *(optional specific exception)*
        - IOException: If an I/O error occurs
        - SecurityException: In the case of the default provider, and a security manager is
                 installed, the SecurityManager.checkRead(String) checkRead
                 method is invoked to check read access to the file.

        Unknown Tags
        - The default implementation is equivalent for this path to:
        ````register(watcher, events, new WatchEvent.Modifier[0]);````
        """
        ...


    def iterator(self) -> Iterator["Path"]:
        """
        Returns an iterator over the name elements of this path.
        
         The first element returned by the iterator represents the name
        element that is closest to the root in the directory hierarchy, the
        second element is the next closest, and so on. The last element returned
        is the name of the file or directory denoted by this path. The .getRoot root component, if present, is not returned by the iterator.

        Returns
        - an iterator over the name elements of this path.

        Unknown Tags
        - The default implementation returns an `Iterator<Path>` which, for
        this path, traverses the `Path`s returned by
        `getName(index)`, where `index` ranges from zero to
        `getNameCount() - 1`, inclusive.
        """
        ...


    def compareTo(self, other: "Path") -> int:
        """
        Compares two abstract paths lexicographically. The ordering defined by
        this method is provider specific, and in the case of the default
        provider, platform specific. This method does not access the file system
        and neither file is required to exist.
        
         This method may not be used to compare paths that are associated
        with different file system providers.

        Arguments
        - other: the path compared to this path.

        Returns
        - zero if the argument is .equals equal to this path, a
                 value less than zero if this path is lexicographically less than
                 the argument, or a value greater than zero if this path is
                 lexicographically greater than the argument

        Raises
        - ClassCastException: if the paths are associated with different providers
        """
        ...


    def equals(self, other: "Object") -> bool:
        """
        Tests this path for equality with the given object.
        
         If the given object is not a Path, or is a Path associated with a
        different `FileSystem`, then this method returns `False`.
        
         Whether or not two path are equal depends on the file system
        implementation. In some cases the paths are compared without regard
        to case, and others are case sensitive. This method does not access the
        file system and the file is not required to exist. Where required, the
        Files.isSameFile isSameFile method may be used to check if two
        paths locate the same file.
        
         This method satisfies the general contract of the java.lang.Object.equals(Object) Object.equals method. 

        Arguments
        - other: the object to which this object is to be compared

        Returns
        - `True` if, and only if, the given object is a `Path`
                 that is identical to this `Path`
        """
        ...


    def hashCode(self) -> int:
        """
        Computes a hash code for this path.
        
         The hash code is based upon the components of the path, and
        satisfies the general contract of the Object.hashCode
        Object.hashCode method.

        Returns
        - the hash-code value for this path
        """
        ...


    def toString(self) -> str:
        """
        Returns the string representation of this path.
        
         If this path was created by converting a path string using the
        FileSystem.getPath getPath method then the path string returned
        by this method may differ from the original String used to create the path.
        
         The returned path string uses the default name FileSystem.getSeparator separator to separate names in the path.

        Returns
        - the string representation of this path
        """
        ...



"""
Python module generated from Java source file java.lang.ref.ReferenceQueue

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.ref import *
from java.util.function import Consumer
from jdk.internal.misc import VM
from typing import Any, Callable, Iterable, Tuple


class ReferenceQueue:

    def __init__(self):
        """
        Constructs a new reference-object queue.
        """
        ...


    def poll(self) -> "Reference"["T"]:
        """
        Polls this queue to see if a reference object is available.  If one is
        available without further delay then it is removed from the queue and
        returned.  Otherwise this method immediately returns `null`.

        Returns
        - A reference object, if one was immediately available,
                 otherwise `null`
        """
        ...


    def remove(self, timeout: int) -> "Reference"["T"]:
        """
        Removes the next reference object in this queue, blocking until either
        one becomes available or the given timeout period expires.
        
         This method does not offer real-time guarantees: It schedules the
        timeout as if by invoking the Object.wait(long) method.

        Arguments
        - timeout: If positive, block for up to `timeout`
                         milliseconds while waiting for a reference to be
                         added to this queue.  If zero, block indefinitely.

        Returns
        - A reference object, if one was available within the specified
                 timeout period, otherwise `null`

        Raises
        - IllegalArgumentException: If the value of the timeout argument is negative
        - InterruptedException: If the timeout wait is interrupted
        """
        ...


    def remove(self) -> "Reference"["T"]:
        """
        Removes the next reference object in this queue, blocking until one
        becomes available.

        Returns
        - A reference object, blocking until one becomes available

        Raises
        - InterruptedException: If the wait is interrupted
        """
        ...



"""
Python module generated from Java source file java.util.Enumeration

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from typing import Any, Callable, Iterable, Tuple


class Enumeration:
    """
    An object that implements the Enumeration interface generates a
    series of elements, one at a time. Successive calls to the
    `nextElement` method return successive elements of the
    series.
    
    For example, to print all elements of a `Vector<E>` *v*:
    ```
      for (Enumeration&lt;E&gt; e = v.elements(); e.hasMoreElements();)
          System.out.println(e.nextElement());```
    
    Methods are provided to enumerate through the elements of a
    vector, the keys of a hashtable, and the values in a hashtable.
    Enumerations are also used to specify the input streams to a
    `SequenceInputStream`.

    Author(s)
    - Lee Boynton

    See
    - java.util.Vector.elements()

    Since
    - 1.0

    Unknown Tags
    - The functionality of this interface is duplicated by the Iterator
    interface.  In addition, `Iterator` adds an optional remove operation,
    and has shorter method names.  New implementations should consider using
    `Iterator` in preference to `Enumeration`. It is possible to
    adapt an `Enumeration` to an `Iterator` by using the
    .asIterator method.
    """

    def hasMoreElements(self) -> bool:
        """
        Tests if this enumeration contains more elements.

        Returns
        - `True` if and only if this enumeration object
                  contains at least one more element to provide;
                 `False` otherwise.
        """
        ...


    def nextElement(self) -> "E":
        """
        Returns the next element of this enumeration if this enumeration
        object has at least one more element to provide.

        Returns
        - the next element of this enumeration.

        Raises
        - NoSuchElementException: if no more elements exist.
        """
        ...


    def asIterator(self) -> Iterator["E"]:
        """
        Returns an Iterator that traverses the remaining elements
        covered by this enumeration. Traversal is undefined if any methods
        are called on this enumeration after the call to `asIterator`.

        Returns
        - an Iterator representing the remaining elements of this Enumeration

        Since
        - 9

        Unknown Tags
        - This method is intended to help adapt code that produces
        `Enumeration` instances to code that consumes `Iterator`
        instances. For example, the java.util.jar.JarFile.entries()
        JarFile.entries() method returns an `Enumeration<JarEntry>`.
        This can be turned into an `Iterator`, and then the
        `forEachRemaining()` method can be used:
        
        ````JarFile jarFile = ... ;
            jarFile.entries().asIterator().forEachRemaining(entry -> { ...`);
        }```
        
        (Note that there is also a java.util.jar.JarFile.stream()
        JarFile.stream() method that returns a `Stream` of entries,
        which may be more convenient in some cases.)
        - The default implementation returns an `Iterator` whose
        Iterator.hasNext hasNext method calls this Enumeration's
        `hasMoreElements` method, whose Iterator.next next
        method calls this Enumeration's `nextElement` method, and
        whose Iterator.remove remove method throws
        `UnsupportedOperationException`.
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.EnderSignal

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Location
from org.bukkit.entity import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class EnderSignal(Entity):
    """
    Represents an EnderSignal, which is created upon throwing an ender eye.
    """

    def getTargetLocation(self) -> "Location":
        """
        Get the location this EnderSignal is moving towards.

        Returns
        - the Location this EnderSignal is moving towards.
        """
        ...


    def setTargetLocation(self, location: "Location") -> None:
        """
        Set the Location this EnderSignal is moving towards.
        
        When setting a new target location, the .getDropItem() resets to
        a random value and the despawn timer gets set back to 0.

        Arguments
        - location: the new target location
        """
        ...


    def getDropItem(self) -> bool:
        """
        Gets if the EnderSignal should drop an item on death.
        If `True`, it will drop an item. If `False`, it will shatter.

        Returns
        - True if the EnderSignal will drop an item on death, or False if
        it will shatter
        """
        ...


    def setDropItem(self, drop: bool) -> None:
        """
        Sets if the EnderSignal should drop an item on death; or if it should
        shatter.

        Arguments
        - drop: True if the EnderSignal should drop an item on death, or
        False if it should shatter.
        """
        ...


    def getItem(self) -> "ItemStack":
        """
        Get the ItemStack to be displayed while in the air and to be
        dropped on death.

        Returns
        - the item stack
        """
        ...


    def setItem(self, item: "ItemStack") -> None:
        """
        Set the ItemStack to be displayed while in the air and to be
        dropped on death.

        Arguments
        - item: the item to set. If null, resets to the default eye of ender
        """
        ...


    def getDespawnTimer(self) -> int:
        """
        Gets the amount of time this entity has been alive (in ticks).
        
        When this number is greater than 80, it will despawn on the next tick.

        Returns
        - the number of ticks this EnderSignal has been alive.
        """
        ...


    def setDespawnTimer(self, timer: int) -> None:
        """
        Set how long this entity has been alive (in ticks).
        
        When this number is greater than 80, it will despawn on the next tick.

        Arguments
        - timer: how long (in ticks) this EnderSignal has been alive.
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.BundleMeta

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.inventory import ItemStack
from org.bukkit.inventory.meta import *
from typing import Any, Callable, Iterable, Tuple


class BundleMeta(ItemMeta):

    def hasItems(self) -> bool:
        """
        Returns whether the item has any items.

        Returns
        - whether items are present
        """
        ...


    def getItems(self) -> list["ItemStack"]:
        """
        Returns an immutable list of the items stored in this item.

        Returns
        - items
        """
        ...


    def setItems(self, items: list["ItemStack"]) -> None:
        """
        Sets the items stored in this item.
        
        Removes all items when given null.

        Arguments
        - items: the items to set
        """
        ...


    def addItem(self, item: "ItemStack") -> None:
        """
        Adds an item to this item.

        Arguments
        - item: item to add
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.raid.RaidStopEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import Raid
from org.bukkit import World
from org.bukkit.event import HandlerList
from org.bukkit.event.raid import *
from typing import Any, Callable, Iterable, Tuple


class RaidStopEvent(RaidEvent):
    """
    Called when a Raid is stopped.
    """

    def __init__(self, raid: "Raid", world: "World", reason: "Reason"):
        ...


    def getReason(self) -> "Reason":
        """
        Returns the stop reason.

        Returns
        - Reason
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class Reason(Enum):

        PEACE = 0
        """
        Because the difficulty has been changed to peaceful.
        """
        TIMEOUT = 1
        """
        The raid took a long time without a final result.
        """
        FINISHED = 2
        """
        Finished the raid.
        """
        UNSPAWNABLE = 3
        """
        Couldn't find a suitable place to spawn raiders.
        """
        NOT_IN_VILLAGE = 4
        """
        The place where the raid occurs no longer be a village.
        """



"""
Python module generated from Java source file org.bukkit.event.HandlerList

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import EnumMap
from java.util import ListIterator
from org.bukkit.event import *
from org.bukkit.plugin import Plugin
from org.bukkit.plugin import RegisteredListener
from typing import Any, Callable, Iterable, Tuple


class HandlerList:
    """
    A list of event handlers, stored per-event. Based on lahwran's fevents.
    """

    def __init__(self):
        """
        Create a new handler list and initialize using EventPriority.
        
        The HandlerList is then added to meta-list for use in bakeAll()
        """
        ...


    @staticmethod
    def bakeAll() -> None:
        """
        Bake all handler lists. Best used just after all normal event
        registration is complete, ie just after all plugins are loaded if
        you're using fevents in a plugin system.
        """
        ...


    @staticmethod
    def unregisterAll() -> None:
        """
        Unregister all listeners from all handler lists.
        """
        ...


    @staticmethod
    def unregisterAll(plugin: "Plugin") -> None:
        """
        Unregister a specific plugin's listeners from all handler lists.

        Arguments
        - plugin: plugin to unregister
        """
        ...


    @staticmethod
    def unregisterAll(listener: "Listener") -> None:
        """
        Unregister a specific listener from all handler lists.

        Arguments
        - listener: listener to unregister
        """
        ...


    def register(self, listener: "RegisteredListener") -> None:
        """
        Register a new listener in this handler list

        Arguments
        - listener: listener to register
        """
        ...


    def registerAll(self, listeners: Iterable["RegisteredListener"]) -> None:
        """
        Register a collection of new listeners in this handler list

        Arguments
        - listeners: listeners to register
        """
        ...


    def unregister(self, listener: "RegisteredListener") -> None:
        """
        Remove a listener from a specific order slot

        Arguments
        - listener: listener to remove
        """
        ...


    def unregister(self, plugin: "Plugin") -> None:
        """
        Remove a specific plugin's listeners from this handler

        Arguments
        - plugin: plugin to remove
        """
        ...


    def unregister(self, listener: "Listener") -> None:
        """
        Remove a specific listener from this handler

        Arguments
        - listener: listener to remove
        """
        ...


    def bake(self) -> None:
        """
        Bake HashMap and ArrayLists to 2d array - does nothing if not necessary
        """
        ...


    def getRegisteredListeners(self) -> list["RegisteredListener"]:
        """
        Get the baked registered listeners associated with this handler list

        Returns
        - the array of registered listeners
        """
        ...


    @staticmethod
    def getRegisteredListeners(plugin: "Plugin") -> list["RegisteredListener"]:
        """
        Get a specific plugin's registered listeners associated with this
        handler list

        Arguments
        - plugin: the plugin to get the listeners of

        Returns
        - the list of registered listeners
        """
        ...


    @staticmethod
    def getHandlerLists() -> list["HandlerList"]:
        """
        Get a list of all handler lists for every event type

        Returns
        - the list of all handler lists
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Vehicle

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from org.bukkit.util import Vector
from typing import Any, Callable, Iterable, Tuple


class Vehicle(Entity):
    """
    Represents a vehicle entity.
    """

    def getVelocity(self) -> "Vector":
        """
        Gets the vehicle's velocity.

        Returns
        - velocity vector
        """
        ...


    def setVelocity(self, vel: "Vector") -> None:
        """
        Sets the vehicle's velocity in meters per tick.

        Arguments
        - vel: velocity vector
        """
        ...



"""
Python module generated from Java source file java.util.Comparator

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import Serializable
from java.util import *
from java.util import Comparators
from java.util.function import Function
from java.util.function import ToDoubleFunction
from java.util.function import ToIntFunction
from java.util.function import ToLongFunction
from typing import Any, Callable, Iterable, Tuple


class Comparator:
    """
    A comparison function, which imposes a *total ordering* on
    some collection of objects.  Comparators can be passed to a sort
    method (such as Collections.sort(List,Comparator)
    Collections.sort or Arrays.sort(Object[],Comparator)
    Arrays.sort) to allow precise control over the sort order.
    Comparators can also be used to control the order of certain data
    structures (such as SortedSet sorted sets or
    SortedMap sorted maps), or to provide an ordering for
    collections of objects that don't have a Comparable
    natural ordering.
    
    The ordering imposed by a comparator `c` on a set of elements
    `S` is said to be *consistent with equals* if and only if
    `c.compare(e1, e2)==0` has the same boolean value as
    `e1.equals(e2)` for every `e1` and `e2` in
    `S`.
    
    Caution should be exercised when using a comparator capable of imposing an
    ordering inconsistent with equals to order a sorted set (or sorted map).
    Suppose a sorted set (or sorted map) with an explicit comparator `c`
    is used with elements (or keys) drawn from a set `S`.  If the
    ordering imposed by `c` on `S` is inconsistent with equals,
    the sorted set (or sorted map) will behave "strangely."  In particular the
    sorted set (or sorted map) will violate the general contract for set (or
    map), which is defined in terms of `equals`.
    
    For example, suppose one adds two elements `a` and `b` such that
    `(a.equals(b) && c.compare(a, b) != 0)`
    to an empty `TreeSet` with comparator `c`.
    The second `add` operation will return
    True (and the size of the tree set will increase) because `a` and
    `b` are not equivalent from the tree set's perspective, even though
    this is contrary to the specification of the
    Set.add Set.add method.
    
    Note: It is generally a good idea for comparators to also implement
    `java.io.Serializable`, as they may be used as ordering methods in
    serializable data structures (like TreeSet, TreeMap).  In
    order for the data structure to serialize successfully, the comparator (if
    provided) must implement `Serializable`.
    
    For the mathematically inclined, the *relation* that defines the
    *imposed ordering* that a given comparator `c` imposes on a
    given set of objects `S` is:```
          {(x, y) such that c.compare(x, y) &lt;= 0}.
    ``` The *quotient* for this total order is:```
          {(x, y) such that c.compare(x, y) == 0}.
    ```
    
    It follows immediately from the contract for `compare` that the
    quotient is an *equivalence relation* on `S`, and that the
    imposed ordering is a *total order* on `S`.  When we say that
    the ordering imposed by `c` on `S` is *consistent with
    equals*, we mean that the quotient for the ordering is the equivalence
    relation defined by the objects' Object.equals(Object)
    equals(Object) method(s):```
        {(x, y) such that x.equals(y)}. ```
    
    In other words, when the imposed ordering is consistent with
    equals, the equivalence classes defined by the equivalence relation
    of the `equals` method and the equivalence classes defined by
    the quotient of the `compare` method are the same.
    
    Unlike `Comparable`, a comparator may optionally permit
    comparison of null arguments, while maintaining the requirements for
    an equivalence relation.
    
    This interface is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.
    
    Type `<T>`: the type of objects that may be compared by this comparator

    Author(s)
    - Neal Gafter

    See
    - java.io.Serializable

    Since
    - 1.2
    """

    def compare(self, o1: "T", o2: "T") -> int:
        """
        Compares its two arguments for order.  Returns a negative integer,
        zero, or a positive integer as the first argument is less than, equal
        to, or greater than the second.
        
        The implementor must ensure that Integer.signum
        signum`(compare(x, y)) == -signum(compare(y, x))` for
        all `x` and `y`.  (This implies that `compare(x, y)` must throw an exception if and only if `compare(y, x)` throws an exception.)
        
        The implementor must also ensure that the relation is transitive:
        `((compare(x, y)>0) && (compare(y, z)>0))` implies
        `compare(x, z)>0`.
        
        Finally, the implementor must ensure that `compare(x,
        y)==0` implies that `signum(compare(x,
        z))==signum(compare(y, z))` for all `z`.

        Arguments
        - o1: the first object to be compared.
        - o2: the second object to be compared.

        Returns
        - a negative integer, zero, or a positive integer as the
                first argument is less than, equal to, or greater than the
                second.

        Raises
        - NullPointerException: if an argument is null and this
                comparator does not permit null arguments
        - ClassCastException: if the arguments' types prevent them from
                being compared by this comparator.

        Unknown Tags
        - It is generally the case, but *not* strictly required that
        `(compare(x, y)==0) == (x.equals(y))`.  Generally speaking,
        any comparator that violates this condition should clearly indicate
        this fact.  The recommended language is "Note: this comparator
        imposes orderings that are inconsistent with equals."
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Indicates whether some other object is &quot;equal to&quot;
        this comparator.  This method must obey the general contract of
        Object.equals(Object).  Additionally, this method can
        return `True` *only* if the specified object is also
        a comparator and it imposes the same ordering as this
        comparator.  Thus, `comp1.equals(comp2)` implies that
        Integer.signum signum`(comp1.compare(o1,
        o2))==signum(comp2.compare(o1, o2))` for every object reference
        `o1` and `o2`.
        
        Note that it is *always* safe *not* to override
        `Object.equals(Object)`.  However, overriding this method may,
        in some cases, improve performance by allowing programs to determine
        that two distinct comparators impose the same order.

        Arguments
        - obj: the reference object with which to compare.

        Returns
        - `True` only if the specified object is also
                 a comparator and it imposes the same ordering as this
                 comparator.

        See
        - Object.hashCode()
        """
        ...


    def reversed(self) -> "Comparator"["T"]:
        """
        Returns a comparator that imposes the reverse ordering of this
        comparator.

        Returns
        - a comparator that imposes the reverse ordering of this
                comparator.

        Since
        - 1.8
        """
        ...


    def thenComparing(self, other: "Comparator"["T"]) -> "Comparator"["T"]:
        """
        Returns a lexicographic-order comparator with another comparator.
        If this `Comparator` considers two elements equal, i.e.
        `compare(a, b) == 0`, `other` is used to determine the order.
        
        The returned comparator is serializable if the specified comparator
        is also serializable.

        Arguments
        - other: the other comparator to be used when this comparator
                compares two objects that are equal.

        Returns
        - a lexicographic-order comparator composed of this and then the
                other comparator

        Raises
        - NullPointerException: if the argument is null.

        Since
        - 1.8

        Unknown Tags
        - For example, to sort a collection of `String` based on the length
        and then case-insensitive natural ordering, the comparator can be
        composed using following code,
        
        ````Comparator<String> cmp = Comparator.comparingInt(String::length)
                    .thenComparing(String.CASE_INSENSITIVE_ORDER);````
        """
        ...


    def thenComparing(self, keyExtractor: "Function"["T", "U"], keyComparator: "Comparator"["U"]) -> "Comparator"["T"]:
        """
        Returns a lexicographic-order comparator with a function that
        extracts a key to be compared with the given `Comparator`.
        
        Type `<U>`: the type of the sort key

        Arguments
        - keyExtractor: the function used to extract the sort key
        - keyComparator: the `Comparator` used to compare the sort key

        Returns
        - a lexicographic-order comparator composed of this comparator
                and then comparing on the key extracted by the keyExtractor function

        Raises
        - NullPointerException: if either argument is null.

        See
        - .thenComparing(Comparator)

        Since
        - 1.8

        Unknown Tags
        - This default implementation behaves as if `thenComparing(comparing(keyExtractor, cmp))`.
        """
        ...


    def thenComparing(self, keyExtractor: "Function"["T", "U"]) -> "Comparator"["T"]:
        """
        Returns a lexicographic-order comparator with a function that
        extracts a `Comparable` sort key.
        
        Type `<U>`: the type of the Comparable sort key

        Arguments
        - keyExtractor: the function used to extract the Comparable sort key

        Returns
        - a lexicographic-order comparator composed of this and then the
                Comparable sort key.

        Raises
        - NullPointerException: if the argument is null.

        See
        - .thenComparing(Comparator)

        Since
        - 1.8

        Unknown Tags
        - This default implementation behaves as if `thenComparing(comparing(keyExtractor))`.
        """
        ...


    def thenComparingInt(self, keyExtractor: "ToIntFunction"["T"]) -> "Comparator"["T"]:
        """
        Returns a lexicographic-order comparator with a function that
        extracts an `int` sort key.

        Arguments
        - keyExtractor: the function used to extract the integer sort key

        Returns
        - a lexicographic-order comparator composed of this and then the
                `int` sort key

        Raises
        - NullPointerException: if the argument is null.

        See
        - .thenComparing(Comparator)

        Since
        - 1.8

        Unknown Tags
        - This default implementation behaves as if `thenComparing(comparingInt(keyExtractor))`.
        """
        ...


    def thenComparingLong(self, keyExtractor: "ToLongFunction"["T"]) -> "Comparator"["T"]:
        """
        Returns a lexicographic-order comparator with a function that
        extracts a `long` sort key.

        Arguments
        - keyExtractor: the function used to extract the long sort key

        Returns
        - a lexicographic-order comparator composed of this and then the
                `long` sort key

        Raises
        - NullPointerException: if the argument is null.

        See
        - .thenComparing(Comparator)

        Since
        - 1.8

        Unknown Tags
        - This default implementation behaves as if `thenComparing(comparingLong(keyExtractor))`.
        """
        ...


    def thenComparingDouble(self, keyExtractor: "ToDoubleFunction"["T"]) -> "Comparator"["T"]:
        """
        Returns a lexicographic-order comparator with a function that
        extracts a `double` sort key.

        Arguments
        - keyExtractor: the function used to extract the double sort key

        Returns
        - a lexicographic-order comparator composed of this and then the
                `double` sort key

        Raises
        - NullPointerException: if the argument is null.

        See
        - .thenComparing(Comparator)

        Since
        - 1.8

        Unknown Tags
        - This default implementation behaves as if `thenComparing(comparingDouble(keyExtractor))`.
        """
        ...


    @staticmethod
    def reverseOrder() -> "Comparator"["T"]:
        """
        Returns a comparator that imposes the reverse of the *natural
        ordering*.
        
        The returned comparator is serializable and throws NullPointerException when comparing `null`.
        
        Type `<T>`: the Comparable type of element to be compared

        Returns
        - a comparator that imposes the reverse of the *natural
                ordering* on `Comparable` objects.

        See
        - Comparable

        Since
        - 1.8
        """
        ...


    @staticmethod
    def naturalOrder() -> "Comparator"["T"]:
        """
        Returns a comparator that compares Comparable objects in natural
        order.
        
        The returned comparator is serializable and throws NullPointerException when comparing `null`.
        
        Type `<T>`: the Comparable type of element to be compared

        Returns
        - a comparator that imposes the *natural ordering* on `Comparable` objects.

        See
        - Comparable

        Since
        - 1.8
        """
        ...


    @staticmethod
    def nullsFirst(comparator: "Comparator"["T"]) -> "Comparator"["T"]:
        """
        Returns a null-friendly comparator that considers `null` to be
        less than non-null. When both are `null`, they are considered
        equal. If both are non-null, the specified `Comparator` is used
        to determine the order. If the specified comparator is `null`,
        then the returned comparator considers all non-null values to be equal.
        
        The returned comparator is serializable if the specified comparator
        is serializable.
        
        Type `<T>`: the type of the elements to be compared

        Arguments
        - comparator: a `Comparator` for comparing non-null values

        Returns
        - a comparator that considers `null` to be less than
                non-null, and compares non-null objects with the supplied
                `Comparator`.

        Since
        - 1.8
        """
        ...


    @staticmethod
    def nullsLast(comparator: "Comparator"["T"]) -> "Comparator"["T"]:
        """
        Returns a null-friendly comparator that considers `null` to be
        greater than non-null. When both are `null`, they are considered
        equal. If both are non-null, the specified `Comparator` is used
        to determine the order. If the specified comparator is `null`,
        then the returned comparator considers all non-null values to be equal.
        
        The returned comparator is serializable if the specified comparator
        is serializable.
        
        Type `<T>`: the type of the elements to be compared

        Arguments
        - comparator: a `Comparator` for comparing non-null values

        Returns
        - a comparator that considers `null` to be greater than
                non-null, and compares non-null objects with the supplied
                `Comparator`.

        Since
        - 1.8
        """
        ...


    @staticmethod
    def comparing(keyExtractor: "Function"["T", "U"], keyComparator: "Comparator"["U"]) -> "Comparator"["T"]:
        """
        Accepts a function that extracts a sort key from a type `T`, and
        returns a `Comparator<T>` that compares by that sort key using
        the specified Comparator.
        
        The returned comparator is serializable if the specified function
        and comparator are both serializable.
        
        Type `<T>`: the type of element to be compared
        
        Type `<U>`: the type of the sort key

        Arguments
        - keyExtractor: the function used to extract the sort key
        - keyComparator: the `Comparator` used to compare the sort key

        Returns
        - a comparator that compares by an extracted key using the
                specified `Comparator`

        Raises
        - NullPointerException: if either argument is null

        Since
        - 1.8

        Unknown Tags
        - For example, to obtain a `Comparator` that compares `Person` objects by their last name ignoring case differences,
        
        ````Comparator<Person> cmp = Comparator.comparing(
                    Person::getLastName,
                    String.CASE_INSENSITIVE_ORDER);````
        """
        ...


    @staticmethod
    def comparing(keyExtractor: "Function"["T", "U"]) -> "Comparator"["T"]:
        """
        Accepts a function that extracts a java.lang.Comparable
        Comparable sort key from a type `T`, and returns a `Comparator<T>` that compares by that sort key.
        
        The returned comparator is serializable if the specified function
        is also serializable.
        
        Type `<T>`: the type of element to be compared
        
        Type `<U>`: the type of the `Comparable` sort key

        Arguments
        - keyExtractor: the function used to extract the Comparable sort key

        Returns
        - a comparator that compares by an extracted key

        Raises
        - NullPointerException: if the argument is null

        Since
        - 1.8

        Unknown Tags
        - For example, to obtain a `Comparator` that compares `Person` objects by their last name,
        
        ````Comparator<Person> byLastName = Comparator.comparing(Person::getLastName);````
        """
        ...


    @staticmethod
    def comparingInt(keyExtractor: "ToIntFunction"["T"]) -> "Comparator"["T"]:
        """
        Accepts a function that extracts an `int` sort key from a type
        `T`, and returns a `Comparator<T>` that compares by that
        sort key.
        
        The returned comparator is serializable if the specified function
        is also serializable.
        
        Type `<T>`: the type of element to be compared

        Arguments
        - keyExtractor: the function used to extract the integer sort key

        Returns
        - a comparator that compares by an extracted key

        Raises
        - NullPointerException: if the argument is null

        See
        - .comparing(Function)

        Since
        - 1.8
        """
        ...


    @staticmethod
    def comparingLong(keyExtractor: "ToLongFunction"["T"]) -> "Comparator"["T"]:
        """
        Accepts a function that extracts a `long` sort key from a type
        `T`, and returns a `Comparator<T>` that compares by that
        sort key.
        
        The returned comparator is serializable if the specified function is
        also serializable.
        
        Type `<T>`: the type of element to be compared

        Arguments
        - keyExtractor: the function used to extract the long sort key

        Returns
        - a comparator that compares by an extracted key

        Raises
        - NullPointerException: if the argument is null

        See
        - .comparing(Function)

        Since
        - 1.8
        """
        ...


    @staticmethod
    def comparingDouble(keyExtractor: "ToDoubleFunction"["T"]) -> "Comparator"["T"]:
        """
        Accepts a function that extracts a `double` sort key from a type
        `T`, and returns a `Comparator<T>` that compares by that
        sort key.
        
        The returned comparator is serializable if the specified function
        is also serializable.
        
        Type `<T>`: the type of element to be compared

        Arguments
        - keyExtractor: the function used to extract the double sort key

        Returns
        - a comparator that compares by an extracted key

        Raises
        - NullPointerException: if the argument is null

        See
        - .comparing(Function)

        Since
        - 1.8
        """
        ...



"""
Python module generated from Java source file java.nio.file.DirectoryStream

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import Closeable
from java.io import IOException
from java.nio.file import *
from java.util import Iterator
from typing import Any, Callable, Iterable, Tuple


class DirectoryStream(Closeable, Iterable):

    def iterator(self) -> Iterator["T"]:
        """
        Returns the iterator associated with this `DirectoryStream`.

        Returns
        - the iterator associated with this `DirectoryStream`

        Raises
        - IllegalStateException: if this directory stream is closed or the iterator has already
                 been returned
        """
        ...


    class Filter:
        """
        An interface that is implemented by objects that decide if a directory
        entry should be accepted or filtered. A `Filter` is passed as the
        parameter to the Files.newDirectoryStream(Path,DirectoryStream.Filter)
        method when opening a directory to iterate over the entries in the
        directory.
        
        Type `<T>`: the type of the directory entry

        Since
        - 1.7
        """

        def accept(self, entry: "T") -> bool:
            """
            Decides if the given directory entry should be accepted or filtered.

            Arguments
            - entry: the directory entry to be tested

            Returns
            - `True` if the directory entry should be accepted

            Raises
            - IOException: If an I/O error occurs
            """
            ...



"""
Python module generated from Java source file com.google.common.base.Enums

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import *
from java.io import Serializable
from java.lang.ref import WeakReference
from java.lang.reflect import Field
from java.util import EnumSet
from java.util import WeakHashMap
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Enums:
    """
    Utility methods for working with Enum instances.

    Author(s)
    - Steve McKay

    Since
    - 9.0
    """

    @staticmethod
    def getField(enumValue: "Enum"[Any]) -> "Field":
        """
        Returns the Field in which `enumValue` is defined. For example, to get the `Description` annotation on the `GOLF` constant of enum `Sport`, use `Enums.getField(Sport.GOLF).getAnnotation(Description.class)`.

        Since
        - 12.0
        """
        ...


    @staticmethod
    def getIfPresent(enumClass: type["T"], value: str) -> "Optional"["T"]:
        """
        Returns an optional enum constant for the given type, using Enum.valueOf. If the
        constant does not exist, Optional.absent is returned. A common use case is for parsing
        user input or falling back to a default enum constant. For example, `Enums.getIfPresent(Country.class, countryInput).or(Country.DEFAULT);`

        Since
        - 12.0
        """
        ...


    @staticmethod
    def stringConverter(enumClass: type["T"]) -> "Converter"[str, "T"]:
        """
        Returns a serializable converter that converts between strings and `enum` values of type
        `enumClass` using Enum.valueOf(Class, String) and Enum.name(). The
        converter will throw an `IllegalArgumentException` if the argument is not the name of any
        enum constant in the specified enum.

        Since
        - 16.0
        """
        ...



"""
Python module generated from Java source file com.google.common.hash.Murmur3_128HashFunction

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.hash import *
from com.google.errorprone.annotations import Immutable
from java.io import Serializable
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Murmur3_128HashFunction(AbstractHashFunction, Serializable):
    """
    See MurmurHash3_x64_128 in <a href="http://smhasher.googlecode.com/svn/trunk/MurmurHash3.cpp">the
    C++ implementation</a>.

    Author(s)
    - Dimitris Andreou
    """

    def bits(self) -> int:
        ...


    def newHasher(self) -> "Hasher":
        ...


    def toString(self) -> str:
        ...


    def equals(self, object: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...



"""
Python module generated from Java source file com.google.common.collect.AbstractIndexedListIterator

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from java.util import ListIterator
from java.util import NoSuchElementException
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class AbstractIndexedListIterator(UnmodifiableListIterator):
    """
    This class provides a skeletal implementation of the ListIterator interface across a
    fixed number of elements that may be retrieved by position. It does not support .remove,
    .set, or .add.

    Author(s)
    - Jared Levy
    """

    def hasNext(self) -> bool:
        ...


    def next(self) -> "E":
        ...


    def nextIndex(self) -> int:
        ...


    def hasPrevious(self) -> bool:
        ...


    def previous(self) -> "E":
        ...


    def previousIndex(self) -> int:
        ...



"""
Python module generated from Java source file org.joml.RayAabIntersection

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class RayAabIntersection:
    """
    This is an implementation of the <a
    href="http://www.cg.cs.tu-bs.de/media/publications/fast-rayaxis-aligned-bounding-box-overlap-tests-using-ray-slopes.pdf">Fast Ray/Axis-Aligned Bounding Box
    Overlap Tests using Ray Slopes</a> paper.
    
    It is an efficient implementation when testing many axis-aligned boxes against the same ray.
    
    This class is thread-safe and can be used in a multithreaded environment when testing many axis-aligned boxes against the same ray concurrently.

    Author(s)
    - Kai Burjack
    """

    def __init__(self):
        """
        Create a new RayAabIntersection without initializing a ray.
        
        Before using the .test(float, float, float, float, float, float) intersect() method,
        the method .set(float, float, float, float, float, float) set() must be called in order to
        initialize the created RayAabIntersection instance with a ray.

        See
        - .set(float, float, float, float, float, float)
        """
        ...


    def __init__(self, originX: float, originY: float, originZ: float, dirX: float, dirY: float, dirZ: float):
        """
        Create a new RayAabIntersection and initialize it with a ray with origin `(originX, originY, originZ)`
        and direction `(dirX, dirY, dirZ)`.
        
        In order to change the direction and/or origin of the ray later, use .set(float, float, float, float, float, float) set().

        Arguments
        - originX: the x coordinate of the origin
        - originY: the y coordinate of the origin
        - originZ: the z coordinate of the origin
        - dirX: the x coordinate of the direction
        - dirY: the y coordinate of the direction
        - dirZ: the z coordinate of the direction

        See
        - .set(float, float, float, float, float, float)
        """
        ...


    def set(self, originX: float, originY: float, originZ: float, dirX: float, dirY: float, dirZ: float) -> None:
        """
        Update the ray stored by this RayAabIntersection with the new origin `(originX, originY, originZ)`
        and direction `(dirX, dirY, dirZ)`.

        Arguments
        - originX: the x coordinate of the ray origin
        - originY: the y coordinate of the ray origin
        - originZ: the z coordinate of the ray origin
        - dirX: the x coordinate of the ray direction
        - dirY: the y coordinate of the ray direction
        - dirZ: the z coordinate of the ray direction
        """
        ...


    def test(self, minX: float, minY: float, minZ: float, maxX: float, maxY: float, maxZ: float) -> bool:
        """
        Test whether the ray stored in this RayAabIntersection intersect the axis-aligned box
        given via its minimum corner `(minX, minY, minZ)` and its maximum corner `(maxX, maxY, maxZ)`.
        
        This implementation uses a tableswitch to dispatch to the correct intersection method.
        
        This method is thread-safe and can be used to test many axis-aligned boxes concurrently.

        Arguments
        - minX: the x coordinate of the minimum corner
        - minY: the y coordinate of the minimum corner
        - minZ: the z coordinate of the minimum corner
        - maxX: the x coordinate of the maximum corner
        - maxY: the y coordinate of the maximum corner
        - maxZ: the z coordinate of the maximum corner

        Returns
        - `True` iff the ray intersects the given axis-aligned box; `False` otherwise
        """
        ...



"""
Python module generated from Java source file org.bukkit.enchantments.Enchantment

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import Lists
from java.util import Locale
from org.bukkit import Keyed
from org.bukkit import NamespacedKey
from org.bukkit import Registry
from org.bukkit import Translatable
from org.bukkit.enchantments import *
from org.bukkit.inventory import ItemStack
from org.bukkit.registry import RegistryAware
from typing import Any, Callable, Iterable, Tuple


class Enchantment(Keyed, Translatable, RegistryAware):
    """
    The various type of enchantments that may be added to armour or weapons
    """

    PROTECTION = getEnchantment("protection")
    """
    Provides protection against environmental damage
    """
    FIRE_PROTECTION = getEnchantment("fire_protection")
    """
    Provides protection against fire damage
    """
    FEATHER_FALLING = getEnchantment("feather_falling")
    """
    Provides protection against fall damage
    """
    BLAST_PROTECTION = getEnchantment("blast_protection")
    """
    Provides protection against explosive damage
    """
    PROJECTILE_PROTECTION = getEnchantment("projectile_protection")
    """
    Provides protection against projectile damage
    """
    RESPIRATION = getEnchantment("respiration")
    """
    Decreases the rate of air loss whilst underwater
    """
    AQUA_AFFINITY = getEnchantment("aqua_affinity")
    """
    Increases the speed at which a player may mine underwater
    """
    THORNS = getEnchantment("thorns")
    """
    Damages the attacker
    """
    DEPTH_STRIDER = getEnchantment("depth_strider")
    """
    Increases walking speed while in water
    """
    FROST_WALKER = getEnchantment("frost_walker")
    """
    Freezes any still water adjacent to ice / frost which player is walking on
    """
    BINDING_CURSE = getEnchantment("binding_curse")
    """
    Item cannot be removed
    """
    SHARPNESS = getEnchantment("sharpness")
    """
    Increases damage against all targets
    """
    SMITE = getEnchantment("smite")
    """
    Increases damage against undead targets
    """
    BANE_OF_ARTHROPODS = getEnchantment("bane_of_arthropods")
    """
    Increases damage against arthropod targets
    """
    KNOCKBACK = getEnchantment("knockback")
    """
    All damage to other targets will knock them back when hit
    """
    FIRE_ASPECT = getEnchantment("fire_aspect")
    """
    When attacking a target, has a chance to set them on fire
    """
    LOOTING = getEnchantment("looting")
    """
    Provides a chance of gaining extra loot when killing monsters
    """
    SWEEPING_EDGE = getEnchantment("sweeping_edge")
    """
    Increases damage against targets when using a sweep attack
    """
    EFFICIENCY = getEnchantment("efficiency")
    """
    Increases the rate at which you mine/dig
    """
    SILK_TOUCH = getEnchantment("silk_touch")
    """
    Allows blocks to drop themselves instead of fragments (for example,
    stone instead of cobblestone)
    """
    UNBREAKING = getEnchantment("unbreaking")
    """
    Decreases the rate at which a tool looses durability
    """
    FORTUNE = getEnchantment("fortune")
    """
    Provides a chance of gaining extra loot when destroying blocks
    """
    POWER = getEnchantment("power")
    """
    Provides extra damage when shooting arrows from bows
    """
    PUNCH = getEnchantment("punch")
    """
    Provides a knockback when an entity is hit by an arrow from a bow
    """
    FLAME = getEnchantment("flame")
    """
    Sets entities on fire when hit by arrows shot from a bow
    """
    INFINITY = getEnchantment("infinity")
    """
    Provides infinite arrows when shooting a bow
    """
    LUCK_OF_THE_SEA = getEnchantment("luck_of_the_sea")
    """
    Decreases odds of catching worthless junk
    """
    LURE = getEnchantment("lure")
    """
    Increases rate of fish biting your hook
    """
    LOYALTY = getEnchantment("loyalty")
    """
    Causes a thrown trident to return to the player who threw it
    """
    IMPALING = getEnchantment("impaling")
    """
    Deals more damage to mobs that live in the ocean
    """
    RIPTIDE = getEnchantment("riptide")
    """
    When it is rainy, launches the player in the direction their trident is thrown
    """
    CHANNELING = getEnchantment("channeling")
    """
    Strikes lightning when a mob is hit with a trident if conditions are
    stormy
    """
    MULTISHOT = getEnchantment("multishot")
    """
    Shoot multiple arrows from crossbows
    """
    QUICK_CHARGE = getEnchantment("quick_charge")
    """
    Charges crossbows quickly
    """
    PIERCING = getEnchantment("piercing")
    """
    Crossbow projectiles pierce entities
    """
    DENSITY = getEnchantment("density")
    """
    Increases fall damage of maces
    """
    BREACH = getEnchantment("breach")
    """
    Reduces armor effectiveness against maces
    """
    WIND_BURST = getEnchantment("wind_burst")
    """
    Emits wind burst upon hitting enemy
    """
    MENDING = getEnchantment("mending")
    """
    Allows mending the item using experience orbs
    """
    VANISHING_CURSE = getEnchantment("vanishing_curse")
    """
    Item disappears instead of dropping
    """
    SOUL_SPEED = getEnchantment("soul_speed")
    """
    Walk quicker on soul blocks
    """
    SWIFT_SNEAK = getEnchantment("swift_sneak")
    """
    Walk quicker while sneaking
    """


    def getName(self) -> str:
        """
        Gets the unique name of this enchantment

        Returns
        - Unique name

        Deprecated
        - enchantments are badly named, use .getKey().
        """
        ...


    def getMaxLevel(self) -> int:
        """
        Gets the maximum level that this Enchantment may become.

        Returns
        - Maximum level of the Enchantment
        """
        ...


    def getStartLevel(self) -> int:
        """
        Gets the level that this Enchantment should start at

        Returns
        - Starting level of the Enchantment
        """
        ...


    def getItemTarget(self) -> "EnchantmentTarget":
        """
        Gets the type of ItemStack that may fit this Enchantment.

        Returns
        - Target type of the Enchantment

        Deprecated
        - enchantment groupings are now managed by tags, not categories
        """
        ...


    def isTreasure(self) -> bool:
        """
        Checks if this enchantment is a treasure enchantment.
        
        Treasure enchantments can only be received via looting, trading, or
        fishing.

        Returns
        - True if the enchantment is a treasure enchantment

        Deprecated
        - enchantment types are now managed by tags
        """
        ...


    def isCursed(self) -> bool:
        """
        Checks if this enchantment is a cursed enchantment
        
        Cursed enchantments are found the same way treasure enchantments are

        Returns
        - True if the enchantment is cursed

        Deprecated
        - cursed enchantments are no longer special. Will return True
        only for Enchantment.BINDING_CURSE and
        Enchantment.VANISHING_CURSE.
        """
        ...


    def conflictsWith(self, other: "Enchantment") -> bool:
        """
        Check if this enchantment conflicts with another enchantment.

        Arguments
        - other: The enchantment to check against

        Returns
        - True if there is a conflict.
        """
        ...


    def canEnchantItem(self, item: "ItemStack") -> bool:
        """
        Checks if this Enchantment may be applied to the given ItemStack.
        
        This does not check if it conflicts with any enchantments already
        applied to the item.

        Arguments
        - item: Item to test

        Returns
        - True if the enchantment may be applied, otherwise False
        """
        ...


    def getKey(self) -> "NamespacedKey":
        """
        See
        - .isRegistered()

        Deprecated
        - A key might not always be present, use .getKeyOrThrow() instead.
        """
        ...


    @staticmethod
    def getByKey(key: "NamespacedKey") -> "Enchantment":
        """
        Gets the Enchantment at the specified key

        Arguments
        - key: key to fetch

        Returns
        - Resulting Enchantment, or null if not found

        Deprecated
        - only for backwards compatibility, use Registry.get(NamespacedKey) instead
        """
        ...


    @staticmethod
    def getByName(name: str) -> "Enchantment":
        """
        Gets the Enchantment at the specified name

        Arguments
        - name: Name to fetch

        Returns
        - Resulting Enchantment, or null if not found

        Deprecated
        - enchantments are badly named, use .getByKey(org.bukkit.NamespacedKey).
        """
        ...


    @staticmethod
    def values() -> list["Enchantment"]:
        """
        Gets an array of all the registered Enchantments

        Returns
        - Array of enchantments

        Deprecated
        - use Registry.iterator() Registry.ENCHANTMENT.iterator()
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.AbstractMapBasedMultimap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.common.collect.Maps import ViewCachingAbstractMap
from com.google.j2objc.annotations import WeakOuter
from java.io import Serializable
from java.util import AbstractCollection
from java.util import Collections
from java.util import Comparator
from java.util import ConcurrentModificationException
from java.util import Iterator
from java.util import ListIterator
from java.util import NavigableMap
from java.util import NavigableSet
from java.util import RandomAccess
from java.util import SortedMap
from java.util import SortedSet
from java.util import Spliterator
from java.util.function import BiConsumer
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class AbstractMapBasedMultimap(AbstractMultimap, Serializable):
    """
    Basic implementation of the Multimap interface. This class represents a multimap as a map
    that associates each key with a collection of values. All methods of Multimap are
    supported, including those specified as optional in the interface.
    
    To implement a multimap, a subclass must define the method .createCollection(), which
    creates an empty collection of values for a key.
    
    The multimap constructor takes a map that has a single entry for each distinct key. When you
    insert a key-value pair with a key that isn't already in the multimap, `AbstractMapBasedMultimap` calls .createCollection() to create the collection of values
    for that key. The subclass should not call .createCollection() directly, and a new
    instance should be created every time the method is called.
    
    For example, the subclass could pass a java.util.TreeMap during construction, and
    .createCollection() could return a java.util.TreeSet, in which case the
    multimap's iterators would propagate through the keys and values in sorted order.
    
    Keys and values may be null, as long as the underlying collection classes support null
    elements.
    
    The collections created by .createCollection() may or may not allow duplicates. If the
    collection, such as a Set, does not support duplicates, an added key-value pair will
    replace an existing pair with the same key and value, if such a pair is present. With collections
    like List that allow duplicates, the collection will keep the existing key-value pairs
    while adding a new pair.
    
    This class is not threadsafe when any concurrent operations update the multimap, even if the
    underlying map and .createCollection() method return threadsafe classes. Concurrent read
    operations will work correctly. To allow concurrent update operations, wrap your multimap with a
    call to Multimaps.synchronizedMultimap.
    
    For serialization to work, the subclass must specify explicit `readObject` and `writeObject` methods.

    Author(s)
    - Louis Wasserman
    """

    def size(self) -> int:
        ...


    def containsKey(self, key: "Object") -> bool:
        ...


    def put(self, key: "K", value: "V") -> bool:
        ...


    def replaceValues(self, key: "K", values: Iterable["V"]) -> Iterable["V"]:
        """
        
        
        The returned collection is immutable.
        """
        ...


    def removeAll(self, key: "Object") -> Iterable["V"]:
        """
        
        
        The returned collection is immutable.
        """
        ...


    def clear(self) -> None:
        ...


    def get(self, key: "K") -> Iterable["V"]:
        """
        
        
        The returned collection is not serializable.
        """
        ...


    def values(self) -> Iterable["V"]:
        """
        
        
        The iterator generated by the returned collection traverses the values for one key, followed
        by the values of a second key, and so on.
        """
        ...


    def entries(self) -> Iterable["Entry"["K", "V"]]:
        """
        
        
        The iterator generated by the returned collection traverses the values for one key, followed
        by the values of a second key, and so on.
        
        Each entry is an immutable snapshot of a key-value mapping in the multimap, taken at the
        time the entry is returned by a method call to the collection or its iterator.
        """
        ...


    def forEach(self, action: "BiConsumer"["K", "V"]) -> None:
        ...



"""
Python module generated from Java source file com.google.common.base.Ticker

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import *
from typing import Any, Callable, Iterable, Tuple


class Ticker:
    """
    A time source; returns a time value representing the number of nanoseconds elapsed since some
    fixed but arbitrary point in time. Note that most users should use Stopwatch instead of
    interacting with this class directly.
    
    **Warning:** this interface can only be used to measure elapsed time, not wall time.

    Author(s)
    - Kevin Bourrillion

    Since
    - 10.0 (<a href="https://github.com/google/guava/wiki/Compatibility">mostly
        source-compatible</a> since 9.0)
    """

    def read(self) -> int:
        """
        Returns the number of nanoseconds elapsed since this ticker's fixed point of reference.
        """
        ...


    @staticmethod
    def systemTicker() -> "Ticker":
        """
        A ticker that reads the current time using System.nanoTime.

        Since
        - 10.0
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.SortedIterables

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from java.util import Comparator
from java.util import SortedSet
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class SortedIterables:
    """
    Utilities for dealing with sorted collections of all types.

    Author(s)
    - Louis Wasserman
    """

    @staticmethod
    def hasSameComparator(comparator: "Comparator"[Any], elements: Iterable[Any]) -> bool:
        """
        Returns `True` if `elements` is a sorted collection using an ordering equivalent to
        `comparator`.
        """
        ...


    @staticmethod
    def comparator(sortedSet: "SortedSet"["E"]) -> "Comparator"["E"]:
        ...



"""
Python module generated from Java source file org.bukkit.event.enchantment.PrepareItemEnchantEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.enchantments import EnchantmentOffer
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.enchantment import *
from org.bukkit.event.inventory import InventoryEvent
from org.bukkit.inventory import ItemStack
from org.bukkit.inventory.view import EnchantmentView
from typing import Any, Callable, Iterable, Tuple


class PrepareItemEnchantEvent(InventoryEvent, Cancellable):
    """
    Called when an ItemStack is inserted in an enchantment table - can be
    called multiple times
    """

    def __init__(self, enchanter: "Player", view: "EnchantmentView", table: "Block", item: "ItemStack", offers: list["EnchantmentOffer"], bonus: int):
        ...


    def getEnchanter(self) -> "Player":
        """
        Gets the player enchanting the item

        Returns
        - enchanting player
        """
        ...


    def getEnchantBlock(self) -> "Block":
        """
        Gets the block being used to enchant the item

        Returns
        - the block used for enchanting
        """
        ...


    def getItem(self) -> "ItemStack":
        """
        Gets the item to be enchanted.

        Returns
        - ItemStack of item
        """
        ...


    def getExpLevelCostsOffered(self) -> list[int]:
        """
        Get a list of offered experience level costs of the enchantment.

        Returns
        - experience level costs offered

        Deprecated
        - Use .getOffers() instead of this method
        """
        ...


    def getOffers(self) -> list["EnchantmentOffer"]:
        """
        Get a list of available EnchantmentOffer for the player. You can
        modify the values to change the available offers for the player. An offer
        may be null, if there isn't a enchantment offer at a specific slot. There
        are 3 slots in the enchantment table available to modify.

        Returns
        - list of available enchantment offers
        """
        ...


    def getEnchantmentBonus(self) -> int:
        """
        Get enchantment bonus in effect - corresponds to number of bookshelves

        Returns
        - enchantment bonus
        """
        ...


    def getView(self) -> "EnchantmentView":
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.representer.SafeRepresenter

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.math import BigInteger
from java.nio.charset import StandardCharsets
from java.util import Arrays
from java.util import Calendar
from java.util import Date
from java.util import Iterator
from java.util import TimeZone
from java.util import UUID
from java.util.regex import Pattern
from org.yaml.snakeyaml import DumperOptions
from org.yaml.snakeyaml.error import YAMLException
from org.yaml.snakeyaml.external.biz.base64Coder import Base64Coder
from org.yaml.snakeyaml.nodes import Node
from org.yaml.snakeyaml.nodes import Tag
from org.yaml.snakeyaml.reader import StreamReader
from org.yaml.snakeyaml.representer import *
from typing import Any, Callable, Iterable, Tuple


class SafeRepresenter(BaseRepresenter):
    """
    Represent standard Java classes
    """

    def __init__(self, options: "DumperOptions"):
        ...


    def addClassTag(self, clazz: type["Object"], tag: "Tag") -> "Tag":
        """
        Define a tag for the `Class` to serialize.

        Arguments
        - clazz: `Class` which tag is changed
        - tag: new tag to be used for every instance of the specified `Class`

        Returns
        - the previous tag associated with the `Class`
        """
        ...


    def getTimeZone(self) -> "TimeZone":
        ...


    def setTimeZone(self, timeZone: "TimeZone") -> None:
        ...



"""
Python module generated from Java source file com.google.common.collect.AbstractSequentialIterator

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from java.util import NoSuchElementException
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class AbstractSequentialIterator(UnmodifiableIterator):
    """
    This class provides a skeletal implementation of the `Iterator` interface for sequences
    whose next element can always be derived from the previous element. Null elements are not
    supported, nor is the .remove() method.
    
    Example:
    
    ````Iterator<Integer> powersOfTwo =
        new AbstractSequentialIterator<Integer>(1) {
          protected Integer computeNext(Integer previous) {
            return (previous == 1 << 30) ? null : previous * 2;`
        };
    }```

    Author(s)
    - Chris Povirk

    Since
    - 12.0 (in Guava as `AbstractLinkedIterator` since 8.0)
    """

    def hasNext(self) -> bool:
        ...


    def next(self) -> "T":
        ...



"""
Python module generated from Java source file net.md_5.bungee.chat.BaseComponentSerializer

Java source file obtained from artifact bungeecord-chat version 1.20-R0.2-20240119.213604-65

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.gson import JsonDeserializationContext
from com.google.gson import JsonElement
from com.google.gson import JsonObject
from com.google.gson import JsonSerializationContext
from java.util import Arrays
from java.util import Collections
from java.util import IdentityHashMap
from java.util import Locale
from net.md_5.bungee.api.chat import BaseComponent
from net.md_5.bungee.api.chat import ClickEvent
from net.md_5.bungee.api.chat import ComponentStyle
from net.md_5.bungee.api.chat import HoverEvent
from net.md_5.bungee.api.chat.hover.content import Content
from net.md_5.bungee.chat import *
from typing import Any, Callable, Iterable, Tuple


class BaseComponentSerializer:





"""
Python module generated from Java source file java.util.concurrent.TimeoutException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.concurrent import *
from typing import Any, Callable, Iterable, Tuple


class TimeoutException(Exception):
    """
    Exception thrown when a blocking operation times out.  Blocking
    operations for which a timeout is specified need a means to
    indicate that the timeout has occurred. For many such operations it
    is possible to return a value that indicates timeout; when that is
    not possible or desirable then `TimeoutException` should be
    declared and thrown.

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def __init__(self):
        """
        Constructs a `TimeoutException` with no specified detail
        message.
        """
        ...


    def __init__(self, message: str):
        """
        Constructs a `TimeoutException` with the specified detail
        message.

        Arguments
        - message: the detail message
        """
        ...
